package pwrace2;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
//import java.util.Scanner;
import java.util.*;

/**
 * This is a second iteration of the pinewood race program.  The data structure
 * is being reorganized so less pointers are handed around.  Operations that
 * work on the data as in sort and create race will be contained in the new
 * structure.  The interface will be the same so no visible differences except
 * a couple of extra options.
 *
 * @author ghewitt
 */

public class Main extends JFrame {
    private JMenuBar menuBar;
    private JMenu fileMenu;
    private JMenu optionMenu;
    private JMenuItem loadPollMenuItem;
    private JMenuItem savePollMenuItem;
    private JMenuItem loadRaceMenuItem;
    private JMenuItem saveRaceMenuItem;
    private JMenuItem saveRaceFormMenuItem;
    private JMenuItem exitMenuItem;
    private JMenuItem optionsMenuItem;
    private JMenuItem createRaceMenuItem;
    private JMenuItem clearRaceMenuItem;
    private JMenuItem viewFullScreenMenuItem;
    private JButton exitFullScreenButton;
    
    private JButton nextEventButton; // non displayed button used by software

    private JPanel mainPanel;
    private JTabbedPane mainTabbedPane;
    private PollPanel pollPanel;
    private RacePanel racePanel;
    private RacePanel racePanelFullScreen;

    private PollArray pollArray;
    private OptionsDialog optionsDialog;
    private JFrame pnt;
    private boolean isFullScreen;
    private GraphicsEnvironment ge;
    private GraphicsDevice gs;
    private FullScreenWindow fullScreenWindow;

    // Constructor of Main Class.  Set up main window and begin execution.
    public Main()  {
        // Set up main window characteristics.
        super ("Pine Wood Derby");
        pnt = this;
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        gs = ge.getDefaultScreenDevice();

        isFullScreen = false;
        pollArray = new PollArray();
        optionsDialog = new OptionsDialog(this);
        
        nextEventButton = new JButton("nextEvent");
        nextEventButton.addActionListener(new NextEventListener());
        

        // Create menu
        viewFullScreenMenuItem = new JMenuItem("View Full Screen");
        if (gs.isFullScreenSupported()) {
            viewFullScreenMenuItem.setEnabled(true);
        } else {
            viewFullScreenMenuItem.setEnabled(false);
        }
        viewFullScreenMenuItem.addActionListener(new ViewFullScreenListener());
        clearRaceMenuItem = new JMenuItem("Clear Race");
        clearRaceMenuItem.setEnabled(false);
        clearRaceMenuItem.addActionListener(new ClearRaceListener());
        createRaceMenuItem = new JMenuItem("Create Race");
        createRaceMenuItem.addActionListener(new CreateRaceListener());
        optionsMenuItem = new JMenuItem("Options");
        optionsMenuItem.addActionListener(new OptionListener());
        exitMenuItem = new JMenuItem("Exit");
        exitMenuItem.addActionListener(new ExitListener());
        saveRaceFormMenuItem = new JMenuItem("Save Form");
        saveRaceFormMenuItem.setEnabled(false);
        saveRaceFormMenuItem.addActionListener(new SaveFormListener());
        saveRaceMenuItem = new JMenuItem("Save Race");
        saveRaceMenuItem.setEnabled(false);
        saveRaceMenuItem.addActionListener(new SaveRaceListener());
        loadRaceMenuItem = new JMenuItem("Load Race");
        loadRaceMenuItem.setEnabled(true);
        loadRaceMenuItem.addActionListener(new LoadRaceListener());
        savePollMenuItem = new JMenuItem("Save Poll");
        savePollMenuItem.addActionListener(new SavePollListener());
        loadPollMenuItem = new JMenuItem("Load Poll");
        loadPollMenuItem.addActionListener(new LoadPollListener());
        optionMenu = new JMenu("Options");
        fileMenu = new JMenu("File");
        menuBar = new JMenuBar();

        fileMenu.add(loadPollMenuItem);
        fileMenu.add(savePollMenuItem);
        fileMenu.add(loadRaceMenuItem);
        fileMenu.add(saveRaceMenuItem);
        fileMenu.add(saveRaceFormMenuItem);
        fileMenu.add(exitMenuItem);
        optionMenu.add(optionsMenuItem);
        optionMenu.add(createRaceMenuItem);
        optionMenu.add(clearRaceMenuItem);
        optionMenu.add(viewFullScreenMenuItem);

        menuBar.add(fileMenu);
        menuBar.add(optionMenu);

        setJMenuBar(menuBar);

        // Create Main Layout
        pollPanel = new PollPanel(pollArray);
        racePanel = new RacePanel(pollArray);
        racePanel.setNextHeatButton(nextEventButton);
        racePanelFullScreen = new RacePanel(pollArray);
        racePanelFullScreen.setNextHeatButton(nextEventButton);

        mainPanel = new JPanel();
        mainPanel.setLayout(new GridLayout(1,1));
        mainTabbedPane = new JTabbedPane();
        mainTabbedPane.addTab("Poll", pollPanel);
        mainTabbedPane.addTab("Race", racePanel);
        mainPanel.add(mainTabbedPane);

        exitFullScreenButton = new JButton("Exit FullScreen");
        exitFullScreenButton.addActionListener(new exitFullScreenListener());

        add(mainPanel);

        // Pack and Display
        pack();
        setVisible(true);
    }



    /**
     * @param args the command line arguments
     */

    // executed method.
    public static void main(String[] args) {
        new Main();
    }

    
    private class LoadPollListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            String tempLine;
            JFileChooser fileChooser = new JFileChooser();
            int status = fileChooser.showOpenDialog(null);
            if(status == JFileChooser.APPROVE_OPTION)
            {
                pollArray.clearEntries();
                File selectedFile = fileChooser.getSelectedFile();
                try
                {
                    Scanner inputFile = new Scanner(selectedFile);
                    tempLine = inputFile.nextLine();
                    if(!tempLine.equals("Pine Wood Derby Poll"))
                    {
                        inputFile.close();
                        throw new IllegalArgumentException("Unknown File type");
                    }
                    while(inputFile.hasNext())
                    {
                        tempLine = inputFile.nextLine();
                        pollArray.addEntry(new PollEntry(tempLine));
                    }
                    inputFile.close();
                }
                catch (Exception err)
                {
                    JOptionPane.showMessageDialog(null, "Error :"
                            + err.getMessage());

                }
                pollArray.fireTableDataChanged();
            }
        }
    }

    private class SavePollListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            String [] tempFileArray = null;
            // copy array to temp File Storage.
            if(!pollArray.isEmpty())  {
                tempFileArray = pollArray.getList();
            }

            JFileChooser fileChooser = new JFileChooser();
            int status = fileChooser.showSaveDialog(null);
            if(status == JFileChooser.APPROVE_OPTION)
            {
                try
                {
                    if(pollArray.isEmpty())
                        throw new IllegalArgumentException("No one in Poll");

                    File selectedFile = fileChooser.getSelectedFile();
                    PrintWriter outputFile = new PrintWriter(selectedFile);

                    outputFile.println("Pine Wood Derby Poll");

                    for(int i=0; i<tempFileArray.length; i++)
                    {
                        outputFile.println(tempFileArray[i]);
                    }
                    outputFile.close();
                }
                catch(Exception err)
                {
                    JOptionPane.showMessageDialog(null, "Error :"
                            + err.getMessage());
                }
            }
        }
    }
    
    private class LoadRaceListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            ArrayList<String> fileStore = new ArrayList<String>();
            fileStore.clear();
            String tempLine, tempEntries[] = null;
            JFileChooser fileChooser = new JFileChooser();
            int status = fileChooser.showOpenDialog(null);
            if(status == JFileChooser.APPROVE_OPTION)
            {
                File selectedFile = fileChooser.getSelectedFile();
                try
                {
                    Scanner inputFile = new Scanner(selectedFile);
                    tempLine = inputFile.nextLine();
                    if(!tempLine.equals("Pine Wood Derby Race"))
                    {
                        inputFile.close();
                        throw new IllegalArgumentException("Unknown File type");
                    }
                    while(inputFile.hasNext())
                    {
                        tempLine = inputFile.nextLine();
                        fileStore.add(tempLine);
                    }
                    inputFile.close();
                }
                catch (Exception err)
                {
                    JOptionPane.showMessageDialog(null, "Error :"
                            + err.getMessage());

                }
            }
            if (!fileStore.isEmpty())  {
                // get poll and place in poll array;
                pollArray.clearEntries();
                int startIndex = 0, endIndex;
                while(!fileStore.get(startIndex).equals("Entries") &&
                        startIndex < fileStore.size() )  {startIndex++;}
                startIndex ++;
                while(startIndex < fileStore.size())  {
                    pollArray.addEntry(new PollEntry(fileStore.get(startIndex)));
                    startIndex++;
                }
                pollArray.fireTableDataChanged();

                // get options and send to options pane
                startIndex = 0;
                while(!fileStore.get(startIndex).equals("Options") &&
                        startIndex < fileStore.size() )  {startIndex++;}
                endIndex = startIndex+1;
                while(!fileStore.get(endIndex).equals("End Options") &&
                        endIndex < fileStore.size())  {endIndex++;}
                tempEntries = new String[endIndex - startIndex + 1];
                for (int i=0; i< tempEntries.length; i++)  {
                    tempEntries[i] = fileStore.get(startIndex+i);
                }
                optionsDialog.setRaceOptions(tempEntries);
                optionsDialog.setRacingStarted(true);
                Color tempFrColor;
                Color tempBkColor;
                racePanel.setNumberOfLanes(optionsDialog.getNumberOfLanes());
                racePanel.setLaneTextFieldsOpaque(optionsDialog.isTextFieldsOpaque());
                racePanel.setUseCarNumColor(optionsDialog.isCarNumUsingCarColors());
                racePanel.setShowPlaceCheckBoxes(optionsDialog.isPlaceBoxesDisplayed());
                racePanel.setShowTimedField(optionsDialog.isTimeFieldDisplayed());

                for(int i=0; i<=optionsDialog.getNumberOfLanes(); i++)  {
                    tempFrColor = optionsDialog.getLaneColor(i)[0];
                    tempBkColor = optionsDialog.getLaneColor(i)[1];
                    racePanel.setLaneColors(i,tempFrColor, tempBkColor);
                }
                racePanel.setRacePanelFont(optionsDialog.getRaceLaneFont());
                racePanel.setLaneItemSpacing(optionsDialog.getLaneItemSpacing());
                racePanel.setLaneBorderSize(optionsDialog.getLaneBorderSize());
                racePanel.setNameDisplay(optionsDialog.getNameDisplayFormat());

                racePanel.regenerate();


                // get racing lineup and send to racing pane
                startIndex = 0;
                while(!fileStore.get(startIndex).equals("Lineup") &&
                        startIndex < fileStore.size() )  {startIndex++;}
                endIndex = startIndex+1;
                while(!fileStore.get(endIndex).equals("End Lineup") &&
                        endIndex < fileStore.size())  {endIndex++;}
                tempEntries = new String[endIndex - startIndex + 1];
                for (int i=0; i< tempEntries.length; i++)  {
                    tempEntries[i] = fileStore.get(startIndex+i);
                }
                racePanel.loadRace(tempEntries);
                saveRaceMenuItem.setEnabled(true);
                saveRaceFormMenuItem.setEnabled(true);
                clearRaceMenuItem.setEnabled(true);
                createRaceMenuItem.setEnabled(false);
                mainTabbedPane.setSelectedIndex(1);
            }
        }
    }
    private class SaveRaceListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            String[] tempFileArray1 = null;
            String[] tempFileArray2 = null;
            String[] tempFileArray3 = null;

            // copy array to temp File Storage.
            tempFileArray1 = optionsDialog.getRaceOptions();
            tempFileArray2 = racePanel.saveRace();
            if(!pollArray.isEmpty())  {
                tempFileArray3 = pollArray.getListAll();
            }

            JFileChooser fileChooser = new JFileChooser();
            int status = fileChooser.showSaveDialog(null);
            if(status == JFileChooser.APPROVE_OPTION)
            {
                try
                {
                    if(pollArray.isEmpty() || tempFileArray2 == null)
                        throw new IllegalArgumentException("No Race to Save");

                    File selectedFile = fileChooser.getSelectedFile();
                    PrintWriter outputFile = new PrintWriter(selectedFile);

                    outputFile.println("Pine Wood Derby Race");

                    for(int i=0; i<tempFileArray1.length; i++)
                    {
                        outputFile.println(tempFileArray1[i]);
                    }
                    outputFile.println(" ");

                    for(int i=0; i<tempFileArray2.length; i++)
                    {
                        outputFile.println(tempFileArray2[i]);
                    }
                    outputFile.println(" ");

                    outputFile.println("Entries");
                    for(int i=0; i<tempFileArray3.length; i++)
                    {
                        outputFile.println(tempFileArray3[i]);
                    }
                    outputFile.close();
                }
                catch(Exception err)
                {
                    JOptionPane.showMessageDialog(null, "Error :"
                            + err.getMessage());
                }
            }
        }
    }
    private class SaveFormListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            String[] tempFileArray1 = null;
            String[] tempFileArray2 = null;
            String[] tempFileArray3 = null;
            String tempLine = null;
            int ln, heats;

            // copy array to temp File Storage.
            tempFileArray1 = racePanel.saveForm();
            if(!pollArray.isEmpty())  {
                tempFileArray3 = pollArray.getListForm();
            }
            ln = optionsDialog.getNumberOfLanes();
            heats = optionsDialog.getNumberOfFinalist();
            tempFileArray2 = new String[heats*3+3];
            tempFileArray2[0] = "Finals Heats";

            for (int i=0; i< heats; i++)  {
                tempLine = "   [      ],______";
                for (int j = 1; j< ln; j++)  {
                    tempLine = tempLine + ", [      ],______";
                }
                tempFileArray2[i+1] = tempLine;
            }
            tempFileArray2[heats+1] = " ";
            tempFileArray2[heats+2] = "Finalists";
            for (int i=0; i< heats; i++)  {
                tempFileArray2[heats+2+i*2] = "______,  _______________________________________";
                tempLine = "   [      ],______";
                for (int j = 1; j< ln; j++)  {
                    tempLine = tempLine + ", [      ],______";
                }
                tempFileArray2[heats+3+i*2] = tempLine;
            }
            tempFileArray2[tempFileArray2.length-1] = "End Heats;";


            JFileChooser fileChooser = new JFileChooser();
            int status = fileChooser.showSaveDialog(null);
            if(status == JFileChooser.APPROVE_OPTION)
            {
                try
                {
                    if(pollArray.isEmpty() || tempFileArray1 == null)
                        throw new IllegalArgumentException("No Race to Save");

                    File selectedFile = fileChooser.getSelectedFile();
                    PrintWriter outputFile = new PrintWriter(selectedFile);

                    outputFile.println("Pine Wood Derby Score Form");

                    for(int i=0; i<tempFileArray1.length; i++)
                    {
                        outputFile.println(tempFileArray1[i]);
                    }
                    outputFile.println(" ");

                    for(int i=0; i<tempFileArray2.length; i++)
                    {
                        outputFile.println(tempFileArray2[i]);
                    }
                    outputFile.println(" ");
                    for(int i=0; i<tempFileArray3.length; i++)
                    {
                        outputFile.println(tempFileArray3[i]);
                    }
                    outputFile.close();
                }
                catch(Exception err)
                {
                    JOptionPane.showMessageDialog(null, "Error :"
                            + err.getMessage());
                }
            }
        }
    }

    private class ExitListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            System.exit(0);
        }
    }

    private class OptionListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            optionsDialog.setVisible(true);
            // update race panel

            Color tempFrColor;
            Color tempBkColor;
            racePanel.setNumberOfLanes(optionsDialog.getNumberOfLanes());
            racePanel.setLaneTextFieldsOpaque(optionsDialog.isTextFieldsOpaque());
            racePanel.setUseCarNumColor(optionsDialog.isCarNumUsingCarColors());
            racePanel.setShowPlaceCheckBoxes(optionsDialog.isPlaceBoxesDisplayed());
            racePanel.setShowTimedField(optionsDialog.isTimeFieldDisplayed());

            for(int i=0; i<=optionsDialog.getNumberOfLanes(); i++)  {
                tempFrColor = optionsDialog.getLaneColor(i)[0];
                tempBkColor = optionsDialog.getLaneColor(i)[1];
                racePanel.setLaneColors(i,tempFrColor, tempBkColor);
            }
            racePanel.setRacePanelFont(optionsDialog.getRaceLaneFont());
            racePanel.setLaneItemSpacing(optionsDialog.getLaneItemSpacing());
            racePanel.setLaneBorderSize(optionsDialog.getLaneBorderSize());
            racePanel.setNameDisplay(optionsDialog.getNameDisplayFormat());

            racePanel.regenerate();
        }
    }

    private class CreateRaceListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            if (!pollArray.isEmpty() && pollArray.getNumberOfMembers() >= optionsDialog.getNumberOfLanes())  {
                pollArray.setRacing();
                mainTabbedPane.setSelectedIndex(1);
                pollPanel.setRaceStarted(true);
                clearRaceMenuItem.setEnabled(true);
                saveRaceMenuItem.setEnabled(true);
                saveRaceFormMenuItem.setEnabled(true);
                createRaceMenuItem.setEnabled(false);
                optionsDialog.setRacingStarted(true);
                pollArray.setRoundRobinStart(optionsDialog.getStartingHeatLineUp());
                int tmpPoll[][];
                pollArray.setRaceType(optionsDialog.getRaceType());
                pollArray.setNumberOfLanes(optionsDialog.getNumberOfLanes());
                pollArray.setRacesPerLane(optionsDialog.getRacesPerLane());
                pollArray.setShuffleHeats(optionsDialog.isShuffledHeats());
                racePanel.setNameDisplay(optionsDialog.getNameDisplayFormat());
                pollArray.setIncludeLeaders(optionsDialog.isLeadersUsed());
                tmpPoll = pollArray.createPreliminaryLineUp();
                if (tmpPoll == null)
                    clearRaceMenuItem.doClick();
                else  {
                    racePanel.startRace(tmpPoll, 0);
                }
            }
            else  {
                if(pollArray.isEmpty())  {
                    JOptionPane.showMessageDialog(null, "The Poll is currently " +
                        "empty", "No Poll", JOptionPane.INFORMATION_MESSAGE);
                }
                else  {
                    JOptionPane.showMessageDialog(null, "Not enough Members " +
                        "in Poll", "No Poll", JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
    }
    private class ClearRaceListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            clearRaceMenuItem.setEnabled(false);
            createRaceMenuItem.setEnabled(true);
            saveRaceMenuItem.setEnabled(false);
            saveRaceFormMenuItem.setEnabled(false);
            racePanel.clearRace();
            pollArray.clearRace();
            pollPanel.setRaceStarted(false);
            optionsDialog.setRacingStarted(false);
        }
    }

    private class ViewFullScreenListener implements ActionListener {
        public void actionPerformed (ActionEvent e) {
            isFullScreen = true;
            fullScreenWindow = new FullScreenWindow();
            exitFullScreenButton.requestFocus();
            pnt.setVisible(false);
            gs.setFullScreenWindow(fullScreenWindow);
            fullScreenWindow.repaint();

        }
    }
    private class NextEventListener implements ActionListener
    {
        public void actionPerformed (ActionEvent e)
        {
            if(isFullScreen)  {
                exitFullScreenButton.doClick();
            }
            int event = racePanel.getCurrentRaceEvent();
            int tPoll[][];
            if(optionsDialog.isSaveRaceData())  {
                saveRaceMenuItem.doClick();
            }

            switch (event)  {
                case 0:  // finished preliminary
                    event = 1;  // change to finals
                    pollArray.setRoundRobinStart(optionsDialog.getFinalsStartingHeatLineUp());
                    pollArray.setRacesPerLane(1);
                    int nf = optionsDialog.getNumberOfFinalist();
                    boolean ucc = optionsDialog.isCarNumUsingCarColors();
                    if (nf > optionsDialog.getNumberOfLanes())  {
                        tPoll = pollArray.createFinalsLineUp(pnt, nf, ucc);
                        if (tPoll != null)  racePanel.startRace(tPoll, 1);
                    }
                    else
                        new WinningDialog(pnt, pollArray.getWinners(), optionsDialog.isCarNumUsingCarColors());
                    break;
                case 1:  // finish finals
                    // display winners
                    new WinningDialog(pnt, pollArray.getWinners(), optionsDialog.isCarNumUsingCarColors());
            }
        }
    }
    
    private class FullScreenWindow extends JFrame  {
        public FullScreenWindow()  {
            super();
            setLayout(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();

            c.weightx = 1;
            c.weighty = 1;
            c.fill = GridBagConstraints.BOTH;
            add(racePanel, c);
            c.gridy = 1;
            c.weightx = 0;
            c.weighty = 0;
            c.anchor = GridBagConstraints.LINE_END;
            c.fill = GridBagConstraints.NONE;
            add(exitFullScreenButton, c);
            racePanel.setRacePanelFont(optionsDialog.getRaceFullScreenLaneFont());
            racePanel.setLaneItemSpacing(optionsDialog.getLaneFullScreenItemSpacing());

            racePanel.regenerate();
        }
    }

    private class exitFullScreenListener implements ActionListener  {

        public void actionPerformed(ActionEvent e) {
            isFullScreen = false;
            pnt.setVisible(true);
            gs.setFullScreenWindow(null);
            mainTabbedPane.addTab("Race", racePanel);
            fullScreenWindow.dispose();

            racePanel.setRacePanelFont(optionsDialog.getRaceLaneFont());
            racePanel.setLaneItemSpacing(optionsDialog.getLaneItemSpacing());

            racePanel.regenerate();
            mainTabbedPane.setSelectedIndex(1);

            validate();
        }
    }
}
