package pwrace2;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.table.AbstractTableModel;
import java.util.ArrayList;
import javax.swing.event.ChangeListener;

/**
 *
 * @author ghewitt
 */

public class OptionsDialog extends JDialog implements WindowListener {
    // public members for retrieval by other classes
    // Race Panel
    private boolean updated;
    private int raceType;
    private int numberOfLanes;
    private int proposedNumOfLanes;
    private boolean placeCheckBoxes;
    private boolean timeField;
    private int racesPerLane;
    private int proposedRacesPerLane;
    private int numberOfFinalist;
    private boolean holdFinalsHeat;
    private boolean randomShuffleHeats;
    private boolean editStartingHeatLineUp;
    private boolean saveRaceData;
    private int[][] oneRaceLineup;
    private int[][] LineUp3_2;
    private int[][] LineUp3_3;
    private int[][] LineUp3_4;
    private int[][] LineUp4_2;
    private int[][] LineUp4_3;
    private int[][] LineUp4_4;
    private int[][] LineUp5_2;
    private int[][] LineUp5_3;
    private int[][] LineUp6_2;

    private Font raceLaneFont; // always plain style
    private Font raceFullScreenFont;
    private int[] laneItemSpacing;  // 5 items, a couple may not be displayed
    private boolean textUseLaneColors;
    private boolean useCarColors;
    private int borderSize;
    private int nameDisplayFormat;
    private boolean useLeaders;
    
    // components in the option dialog
    private String raceTypeListString[];  // string array for list
    private String fontTypeListString[];
    private String nameDisplayListString[];

    // Display components
    private JPanel optionsPanel;
    private JPanel buttonPanel;
    private JTabbedPane dialogTabbedPane;
    private JPanel displayOptionsPanel;
    private JPanel laneSpacingPanel;
    private JPanel raceOptionsPanel;
    private RRobinPanel roundRobinCalcPanel;
    private JButton okButton;
    private JButton cancelButton;

    private JComboBox raceFontComboBox;
    private JTextField raceFontSizeTextField;
    private JTextField raceFullScreenFontSizeTextField;
    private JCheckBox raceBoldCheckBox;
    private JCheckBox raceFullScreenBoldCheckBox;
    private JCheckBox useLaneColorsCheckBox;
    private JCheckBox useCarColorsCheckBox;
    private JComboBox nameDisplayComboBox;
    private JTextField laneBorderSizeTextField;
    private JTextField[] laneSpacingTextFields;
    private JButton loadDefaultRRButton;

    private JTable laneColorTable;
    private JScrollPane laneColorScrollPane;
    private LaneColorTableModel laneColorTableModel;

    private JTextField numberOfLanesTextField;
    private JTextField racesPerLaneTextField;
    private JTextField numberOfFinalistTextField;
    private JComboBox raceTypeComboBox;
    private JCheckBox displayPlaceCheckBox;
    private JCheckBox holdFinalsHeatCheckBox;
    private JCheckBox shuffleHeatCheckBox;
    private JCheckBox useLeadersCheckBox;
    private JCheckBox saveRaceDataCheckBox;

    private StartingHeatTableModel startingHeatTableModel;
    private JTable startingHeatTable;
    private JScrollPane startingHeatScrollPane;

    public OptionsDialog (JFrame parent)  {
        // initialize to default values.
        // may eventually load values from file
        // use hard coded values if no file exists
        super(parent, "Options", true);
        addWindowListener(this);
        setMinimumSize(new Dimension(420, 400));

        // race Values;
        updated = false;
        numberOfLanes = 4;
        proposedNumOfLanes = 4;
        raceType=0;
        timeField = false;
        racesPerLane = 2;
        proposedRacesPerLane = 2;
        placeCheckBoxes = true;
        numberOfFinalist = 10;
        holdFinalsHeat = true;
        randomShuffleHeats = false;
        editStartingHeatLineUp = false;
        nameDisplayFormat = 0;
        setUpRaceArrays();

        // display Values;
        raceTypeListString = new String[7];
        raceTypeListString[0] = "Chaotic Rotation (RR Double Shuffle)";
        raceTypeListString[1] = "Simple Chaotic Rotation";
        raceTypeListString[2] = "Lane rotation";
        raceTypeListString[3] = "Round Robin";
        raceTypeListString[4] = "Timed Race, Chaotic Lineup";
        raceTypeListString[5] = "Timed Race, Simple Chaotic Rotation";
        raceTypeListString[6] = "Timed Race, Round Robin Lineup";

        fontTypeListString = new String[5];
        fontTypeListString[0] = "Dialog";
        fontTypeListString[1] = "DialogInput";
        fontTypeListString[2] = "Monospaced";
        fontTypeListString[3] = "Serif";
        fontTypeListString[4] = "SansSerif";
        
        nameDisplayListString = new String[3];
        nameDisplayListString[0] = "First Last";
        nameDisplayListString[1] = "F. Last";
        nameDisplayListString[2] = "First L.";


        raceLaneFont = new Font("Dialog", Font.BOLD, 18);
        raceFullScreenFont = new Font("Dialog", Font.BOLD, 24);

        raceType = 0;
        numberOfLanes = 4;
        laneItemSpacing = new int[12];
        laneItemSpacing[0] = 50;
        laneItemSpacing[1] = 50;
        laneItemSpacing[2] = 400;
        laneItemSpacing[3] = 100;
        laneItemSpacing[4] = 200;
        laneItemSpacing[5] = 40; // height
        laneItemSpacing[6] = 80;
        laneItemSpacing[7] = 80;
        laneItemSpacing[8] = 500;
        laneItemSpacing[9] = 150;
        laneItemSpacing[10] = 300;
        laneItemSpacing[11] = 75;

        textUseLaneColors = false;
        useCarColors = true;
        borderSize = 10;

        // Display panel;
        optionsPanel = new JPanel();
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
        optionsPanel.setLayout(new GridBagLayout());

        displayOptionsPanel = new JPanel();
        displayOptionsPanel.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(3,3,3,3);

        c.gridx = 0;
        c.gridy = 0;
        displayOptionsPanel.add(new JLabel("Font:"), c);
      
        raceFontComboBox = new JComboBox(fontTypeListString);
        c.gridx = 1;
        c.gridy = 0;
        displayOptionsPanel.add(raceFontComboBox, c);
       
        c.gridx = 2;
        c.gridy = 0;
        c.anchor = GridBagConstraints.LINE_END;
        displayOptionsPanel.add(new JLabel("Size:"), c);

        raceFontSizeTextField = new JTextField(3);
        c.gridx = 3;
        c.gridy = 0;
        c.anchor = GridBagConstraints.LINE_START;
        displayOptionsPanel.add(raceFontSizeTextField, c);
        raceFontSizeTextField.setText("18");

        c.gridx = 1;
        c.gridy = 1;
        c.gridwidth = 2;
        c.anchor = GridBagConstraints.LINE_END;
        displayOptionsPanel.add(new JLabel ("Full Screen Size:"), c);

        raceFullScreenFontSizeTextField = new JTextField (3);
        c.gridx = 3;
        c.gridy = 1;
        c.gridwidth = 1;
        c.anchor = GridBagConstraints.LINE_START;
        displayOptionsPanel.add(raceFullScreenFontSizeTextField, c);
        raceFullScreenFontSizeTextField.setText("24");

        raceBoldCheckBox = new JCheckBox("Bold", true);
        c.gridx = 4;
        c.gridy = 0;
        displayOptionsPanel.add(raceBoldCheckBox, c);

        raceFullScreenBoldCheckBox = new JCheckBox("Bold", true);
        c.gridx = 4;
        c.gridy = 1;
        displayOptionsPanel.add(raceFullScreenBoldCheckBox, c);

        laneColorTableModel = new LaneColorTableModel();
        laneColorTable = new JTable(laneColorTableModel);
        laneColorTable.setFillsViewportHeight(true);
        laneColorTable.setDefaultRenderer(Color.class, new ColorRenderer(true));
        laneColorTable.setDefaultEditor(Color.class, new ColorEditor());
        laneColorTable.setDefaultRenderer(JLabel.class, new CarNumberRenderer(true));
        laneColorTable.setAutoCreateRowSorter(true);
        laneColorTable.setColumnSelectionAllowed(false);
        int[]columnWidths = { 30, 50, 50};
        for (int i = 0; i < 3; i++) {
            laneColorTable.getColumnModel().getColumn(i).
                    setPreferredWidth(columnWidths[i]);
        }
        laneColorScrollPane = new JScrollPane(laneColorTable);
        laneColorScrollPane.setPreferredSize(new Dimension(150, 100));
        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth = 2;
        c.gridheight = 4;
        c.fill = GridBagConstraints.BOTH;
        displayOptionsPanel.add(laneColorScrollPane, c);
        c.fill = GridBagConstraints.NONE;

        useLaneColorsCheckBox = new JCheckBox("Text Fields use lane colors");
        c.gridx = 2;
        c.gridwidth = 3;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.LINE_START;

        displayOptionsPanel.add(useLaneColorsCheckBox, c);

        useCarColorsCheckBox =
                new JCheckBox("Display car number in car colors", true);
        c.gridy = 3;
        displayOptionsPanel.add(useCarColorsCheckBox, c);
        
        c.gridx = 2;
        c.gridy = 4;
        c.gridwidth = 2;
        displayOptionsPanel.add(new JLabel("Lane Border Size:"),c);

        laneBorderSizeTextField = new JTextField(3);
        laneBorderSizeTextField.setText("10");
        c.gridx = 4;
        c.gridy = 4;
        c.gridwidth = 1;
        displayOptionsPanel.add(laneBorderSizeTextField,c);

        c.gridx = 2;
        c.gridy = 5;
        c.gridwidth = 2;
        c.weighty = 1;
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        displayOptionsPanel.add(new JLabel("Display name as: "),c);
        c.weighty = 0;

        nameDisplayComboBox = new JComboBox(nameDisplayListString);
        nameDisplayComboBox.setSelectedIndex(nameDisplayFormat);
        c.gridx = 4;
        c.gridy = 5;
//        c.gridwidth = 2;
        c.weightx = 1;
        displayOptionsPanel.add(nameDisplayComboBox, c);
        c.anchor = GridBagConstraints.LINE_START;
        c.weightx = 0;
        
        c.gridx = 0;
        c.gridy = 6;
        c.gridwidth = 3;
        displayOptionsPanel.add(new JLabel("Prefered Spacing of Lane Fields"),c);

        laneSpacingPanel = new JPanel();
        laneSpacingPanel.setLayout(new GridBagLayout());
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.LINE_END;

        laneSpacingTextFields = new JTextField[12];
        for (int i=0; i<12; i++) {
            laneSpacingTextFields[i] = new JTextField(3);
            laneSpacingTextFields[i].setText(""+laneItemSpacing[i]);
        }

        c.gridy = 1;
        laneSpacingPanel.add(new JLabel("Normal"),c);
        c.gridy = 2;
        laneSpacingPanel.add(new JLabel("Full Screen"),c);

        c.gridy = 0;
        c.anchor = GridBagConstraints.LINE_START;
        c.gridx = 1;
        laneSpacingPanel.add(new JLabel("Lane"),c);
        c.gridx = 2;
        laneSpacingPanel.add(new JLabel("Car #"),c);
        c.gridx = 3;
        laneSpacingPanel.add(new JLabel("Driver"),c);
        c.gridx = 4;
        laneSpacingPanel.add(new JLabel("Time"),c);
        c.gridx = 5;
        laneSpacingPanel.add(new JLabel("Place"),c);
        c.gridx = 6;
        laneSpacingPanel.add(new JLabel("Height"),c);

        for (int j=0; j<2; j++)
            for (int i=0; i<6; i++)  {
                c.gridy = j+1;
                c.gridx = i+1;
                laneSpacingPanel.add(laneSpacingTextFields[i+j*6],c);
            }

        c.gridx = 0;
        c.gridy = 7;
        c.gridwidth = 5;
        displayOptionsPanel.add(laneSpacingPanel, c);

        raceOptionsPanel = new JPanel();
        raceOptionsPanel.setLayout(new GridBagLayout());
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        raceOptionsPanel.add(new JLabel("Number Of Lanes:"), c);
        c.gridy = 1;
        raceOptionsPanel.add(new JLabel("Races per Lane:"), c);
        c.gridy = 2;
        raceOptionsPanel.add(new JLabel("Race Shuffle Type:"), c);
        c.gridy = 3;
        raceOptionsPanel.add(new JLabel("Number of Finalists:"), c);
        c.gridy = 6;
        raceOptionsPanel.add(new JLabel("Starting Heat Line up:"), c);

        c.gridx = 2;
        loadDefaultRRButton = new JButton("Load Default");
        loadDefaultRRButton.addActionListener(new LoadDefaultActionListener());
        raceOptionsPanel.add(loadDefaultRRButton, c);

        numberOfLanesTextField = new JTextField(3);
        numberOfLanesTextField.addFocusListener(new numberOfLanesFocusListener());
        c.gridy = 0;
        c.gridx = 1;
        numberOfLanesTextField.setText("4");
        raceOptionsPanel.add(numberOfLanesTextField, c);

        racesPerLaneTextField = new JTextField(3);
        racesPerLaneTextField.addFocusListener(new RacesPerLaneFocusListener());
        c.gridy = 1;
        racesPerLaneTextField.setText("2");
        raceOptionsPanel.add(racesPerLaneTextField, c);

        raceTypeComboBox = new JComboBox(raceTypeListString);
        raceTypeComboBox.addActionListener(new RaceTypeActionListener());
        c.gridy = 2;
        c.gridwidth = 2;
        raceOptionsPanel.add(raceTypeComboBox, c);
        c.gridwidth = 1;

        numberOfFinalistTextField = new JTextField(3);
        c.gridy = 3;
        numberOfFinalistTextField.setText("10");
        raceOptionsPanel.add(numberOfFinalistTextField, c);

        displayPlaceCheckBox = new JCheckBox("Display Place Check boxes");
        displayPlaceCheckBox.setSelected(placeCheckBoxes);
        c.gridx = 0;
        c.gridy = 4;
        c.gridwidth = 2;
        raceOptionsPanel.add(displayPlaceCheckBox, c);
        c.gridwidth = 1;

        holdFinalsHeatCheckBox = new JCheckBox("Conduct Final's Heats");
        holdFinalsHeatCheckBox.setSelected(holdFinalsHeat);
        c.gridx = 2;
        c.gridy = 3;
        raceOptionsPanel.add(holdFinalsHeatCheckBox, c);


        useLeadersCheckBox = new JCheckBox("Use Leaders in Preliminary Heats");
        useLeadersCheckBox.setSelected(useLeaders);
        c.gridx = 2;
        c.gridy = 4;
        c.gridwidth = 1;
        c.weightx = 1;
        raceOptionsPanel.add(useLeadersCheckBox, c);
        c.weightx = 0;

        shuffleHeatCheckBox = new JCheckBox("Randomly Shuffle Heats");
        shuffleHeatCheckBox.setSelected(randomShuffleHeats);
        c.gridx = 2;
        c.gridy = 5;
        raceOptionsPanel.add(shuffleHeatCheckBox, c);

        saveRaceDataCheckBox = new JCheckBox("Save Race Data after Event");
        c.gridx = 0;
        c.gridy = 5;
        c.gridwidth = 2;
        raceOptionsPanel.add(saveRaceDataCheckBox, c);
        c.gridwidth = 1;

        startingHeatTableModel = new StartingHeatTableModel();
        startingHeatTable = new JTable(startingHeatTableModel);
        startingHeatTable.setColumnSelectionAllowed(false);
        startingHeatTable.setRowSelectionAllowed(false);
        startingHeatTable.setPreferredScrollableViewportSize(new Dimension(350, 75));

        startingHeatScrollPane = new JScrollPane(startingHeatTable);
        startingHeatScrollPane.setPreferredSize(new Dimension(350, 75));
        startingHeatScrollPane.setEnabled(false);
        c.gridx = 0;
        c.gridy = 7;
        c.gridwidth = 3;
        c.weighty = 1;
        c.fill = GridBagConstraints.BOTH;
        raceOptionsPanel.add(startingHeatScrollPane, c);
        c.weighty = 0;
        c.fill = GridBagConstraints.NONE;

        roundRobinCalcPanel = new RRobinPanel();
        dialogTabbedPane = new JTabbedPane();
        dialogTabbedPane.add("Display Options",displayOptionsPanel);
        dialogTabbedPane.add("Race Options", raceOptionsPanel);
        dialogTabbedPane.add("Round Robin Calc", roundRobinCalcPanel);
        dialogTabbedPane.addChangeListener(new tabChangeListener());

        okButton = new JButton("OK");
        okButton.addActionListener(new OkButtonActionListener());
        
        cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new CancelButtonActionListener());

        c = new GridBagConstraints();  // reset to defaults
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;
        c.weighty = 1;
        c.fill = GridBagConstraints.BOTH;
        optionsPanel.add(dialogTabbedPane, c);
        c.gridy = 1;
        c.weightx = 0;
        c.weighty = 0;
        c.fill = GridBagConstraints.HORIZONTAL;
        buttonPanel.add(okButton);
        buttonPanel.add(cancelButton);
        optionsPanel.add(buttonPanel, c);
        add(optionsPanel);
        raceTypeComboBox.setSelectedIndex(0);
        raceFontComboBox.setSelectedIndex(0);
        pack();
    }

    public boolean isUpdated() {
        return updated;
    }
    public void setRaceOptions(String[] optionsString)  {
        // number of lanes, races per lane, Starting lineup,
        // number of finalists, race type, shuffle heats

        String[] optionsData = null;
        int [][] lu = null;
        int i=0;
        // set actual values, then click cancel to set displays
        // note soptionsString[0] just heater "Options Settings"
        try {
            optionsData = optionsString[1].split(",");
            for(i=0;i<optionsData.length;i++)
                optionsData[i] = optionsData[i].trim();
            numberOfLanes = Integer.parseInt(optionsData[1]);
            optionsData = optionsString[2].split(",");
            for(i=0;i<optionsData.length;i++)
                optionsData[i] = optionsData[i].trim();
            racesPerLane = Integer.parseInt(optionsData[1]);
            optionsData = optionsString[3].split(",");
            for(i=0;i<optionsData.length;i++)
                optionsData[i] = optionsData[i].trim();
            holdFinalsHeat = true;
            numberOfFinalist = Integer.parseInt(optionsData[1]);
            if (numberOfFinalist == -1)  {
                holdFinalsHeat = false;
                numberOfFinalist = 10;
            }
            optionsData = optionsString[4].split(",");
            for(i=0;i<optionsData.length;i++)
                optionsData[i] = optionsData[i].trim();
            raceType = Integer.parseInt(optionsData[1]);
            optionsData = optionsString[5].split(",");
            for(i=0;i<optionsData.length;i++)
                optionsData[i] = optionsData[i].trim();
            randomShuffleHeats = Boolean.parseBoolean(optionsData[1]);
            
            lu = new int[racesPerLane+1][numberOfLanes];
            // races per lane in preliminary heats,  plus final heats
            for (int row = 0; row < racesPerLane+1; row++ )  {
                optionsData = optionsString[row+7].split(",");
                for(i=0;i<optionsData.length;i++)
                    optionsData[i] = optionsData[i].trim();
                for (int col = 0; col<numberOfLanes; col++)  {
                    lu[row][col] = Integer.parseInt(optionsData[col+1]);
                }
            }
            startingHeatTableModel.setAllLineUps(lu);

            // load lane colors
            int index = racesPerLane+8;
            String lnCl[] = new String[numberOfLanes+1];
            for (int row=0; row < numberOfLanes+1; row++)  {
                lnCl[row] = optionsString[index+row];
            }
            laneColorTableModel.loadLaneColors(lnCl);

            // Click cancel to update displays
            cancelButton.doClick();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Error Parsing Options"
                    + e.getMessage());
        }
    }
    
    public String[] getRaceOptions()  {
        int lu[][];
        int index = 0;
        String[] saveColors = laneColorTableModel.saveLaneColors();
        String temp = null;
        String[] OptionsString = new String[racesPerLane + saveColors.length + 9];
        OptionsString[0] = "Options";
        OptionsString[OptionsString.length-1] = "End Options";
        OptionsString[1] = "Number Of Lanes, " + numberOfLanes;
        OptionsString[2] = "Races Per Lane, " + racesPerLane;
        if(holdFinalsHeat)
            OptionsString[3] = "Number Of Finalists, " + numberOfFinalist;
        else
            OptionsString[3] = "Number Of Finalists, " + (-1);
        OptionsString[4] = "Race Type, " + raceType;
        OptionsString[5] = "Randomly Shuffle Heats, " + randomShuffleHeats;
        OptionsString[6] = "Starting Heat Lineup:";
        lu = startingHeatTableModel.saveAllLineUps();
        for (int i=0; i< lu.length; i++)  {
            temp = "" + lu[i][0];
            for (int j=1; j< lu[0].length; j++)  {
                temp = temp + "," + lu[i][j];
            }
            OptionsString[i+7] = temp;
        }
        index = lu.length + 7;

        for(int i=0; i<saveColors.length; i++)  {
            OptionsString[index+i] = saveColors[i];
        }

        return OptionsString;
    }

    // get display options values
    public Font getRaceLaneFont()  {
        return raceLaneFont;
    }
    public Font getRaceFullScreenLaneFont()  {
        return raceFullScreenFont;
    }
    public int[] getLaneItemSpacing()  {
        int [] temp = new int[6];
        for (int i=0; i<6; i++)  {
            temp[i] = laneItemSpacing[i];
        }
        return temp;
    }
    public int[] getLaneFullScreenItemSpacing()  {
        int [] temp = new int[6];
        for (int i=0; i<6; i++)  {
            temp[i] = laneItemSpacing[i+6];
        }
        return temp;
    }
    public boolean isPlaceBoxesDisplayed()  {
        return placeCheckBoxes;
    }
    public boolean isCarNumUsingCarColors()  {
        return useCarColors;
    }
    public boolean isTextFieldsOpaque()  {
        return !textUseLaneColors;
    }
    public boolean isShuffledHeats()  {
        return randomShuffleHeats;
    }
    public boolean isLeadersUsed()  {
        return useLeaders;
    }
    public boolean isSaveRaceData()  {
        return saveRaceData;
    }
    public int getLaneBorderSize()  {
        return borderSize;
    }
    public Color[] getLaneColor(int ln) {
        return laneColorTableModel.getLaneColors(ln);
    }
    public boolean isTimeFieldDisplayed()  {
        return timeField;
    }

    public int getNameDisplayFormat()  {
        return nameDisplayFormat;
    }

    // get race options values
    public int getNumberOfLanes() {
        return numberOfLanes;
    }
    public int getRacesPerLane()  {
        return racesPerLane;
    }
    public int getRaceType()  {
        return raceType;
    }
    public int getNumberOfFinalist()  {
        int tempFinals;
        if (!holdFinalsHeat) tempFinals = -1;
        else tempFinals = numberOfFinalist;
        return tempFinals;
    }

    public int[][] getStartingHeatLineUp()  {
        return startingHeatTableModel.getPreliminaryHeatLineUp();
    }
    public int[][] getFinalsStartingHeatLineUp()  {
        return startingHeatTableModel.getFinalsHeatLineUp();
    }
    public void setRacingStarted(boolean raceStarted)  {
        if (raceStarted)  {
            // disabling race options
            numberOfLanesTextField.setEnabled(false);
            raceTypeComboBox.setEnabled(false);
            racesPerLaneTextField.setEnabled(false);
            numberOfFinalistTextField.setEnabled(false);
            holdFinalsHeatCheckBox.setEnabled(false);
            editStartingHeatLineUp = false;
            useLeadersCheckBox.setEnabled(false);
            dialogTabbedPane.setSelectedIndex(0);
            dialogTabbedPane.setEnabledAt(2, false);
            startingHeatTable.setEnabled(false);
            loadDefaultRRButton.setEnabled(false);
        }
        else {
            // enable race options
            numberOfLanesTextField.setEnabled(true);
            racesPerLaneTextField.setEnabled(true);
            numberOfFinalistTextField.setEnabled(true);
            holdFinalsHeatCheckBox.setEnabled(true);
            useLeadersCheckBox.setEnabled(true);
            raceTypeComboBox.setEnabled(true);
            raceTypeComboBox.setSelectedIndex(raceType);
            dialogTabbedPane.setEnabledAt(2, true);
            loadDefaultRRButton.setEnabled(true);
        }
    }

    private class LaneRowColors{
        public JLabel laneLabel;
        public Color[] rowColors;
        public LaneRowColors() {
            rowColors = new Color[2];
            rowColors[0] = Color.BLACK;
            rowColors[1] = Color.WHITE;
            laneLabel = new JLabel();
            laneLabel.setForeground(Color.BLACK);
            laneLabel.setBackground(Color.WHITE);
        }
        public LaneRowColors(int row, Color c1, Color c2)  {
            rowColors = new Color[2];
            rowColors[0] = c1;
            rowColors[1] = c2;
            laneLabel = new JLabel();
            
            if (row <1)  laneLabel.setText("Title");
            else laneLabel.setText("" + row + ".");

            laneLabel.setForeground(c1);
            laneLabel.setBackground(c2);
        }
        public LaneRowColors getCopy()  {
            LaneRowColors tempColors = new LaneRowColors(0, rowColors[0], rowColors[1]);
            tempColors.laneLabel.setText(laneLabel.getText());
            return tempColors;
        }
    }

    private class LaneColorTableModel extends AbstractTableModel  {

        private String[] columnNames;
        private ArrayList<LaneRowColors> laneColorsArray;
        private ArrayList<LaneRowColors> proposedColorArray;

        private LaneColorTableModel()  {
            columnNames = new String[3];
            columnNames[0] = "Lane";
            columnNames[1] = "Foreground";
            columnNames[2] = "Background";
            laneColorsArray = new ArrayList<LaneRowColors>();
            laneColorsArray.add(new LaneRowColors(0,Color.BLACK,Color.WHITE));
            laneColorsArray.add(new LaneRowColors(1,Color.BLACK,Color.YELLOW));
            laneColorsArray.add(new LaneRowColors(2,Color.BLACK,Color.GREEN));
            laneColorsArray.add(new LaneRowColors(3,Color.WHITE,Color.BLUE));
            laneColorsArray.add(new LaneRowColors(4,Color.WHITE,Color.RED));

            copyToProposedArray();
        }

        public Color[] getLaneColors(int lane)  {
            Color[] tempColors = new Color[2];
            if(lane < 0 || lane > numberOfLanes)  lane = 0;
            tempColors[0] = laneColorsArray.get(lane).rowColors[0];
            tempColors[1] = laneColorsArray.get(lane).rowColors[1];
            return tempColors;
        }
        public String[] saveLaneColors()  {
            String[] temp = new String[numberOfLanes+1];
            Color[] tc;
            for (int ln = 0; ln <= numberOfLanes; ln ++)  {
                tc = getLaneColors(ln);
                temp[ln] = ""+tc[0].getRed()+","+tc[0].getGreen()+","+tc[0].getBlue()+
                        ","+tc[1].getRed()+","+tc[1].getGreen()+","+tc[1].getBlue();
            }

            return temp;
        }
        public void loadLaneColors(String[] tc)  {
            String[] temp;
            int r1, g1, b1, r2, g2, b2;
            laneColorsArray.clear();
            try {
                for (int ln = 0; ln <= numberOfLanes; ln++) {
                    temp = tc[ln].split(",");
                    for (int j=0; j< temp.length; j++)
                        temp[j] = temp[j].trim();
                    r1 = Integer.parseInt(temp[0]);
                    g1 = Integer.parseInt(temp[1]);
                    b1 = Integer.parseInt(temp[2]);
                    r2  = Integer.parseInt(temp[3]);
                    g2 = Integer.parseInt(temp[4]);
                    b2 = Integer.parseInt(temp[5]);
                    laneColorsArray.add(new LaneRowColors(ln, new Color(r1,g1,b1), new Color(r2, g2, b2)));
                }
            } catch (Exception e) {
            }
        }
        
        public void updateLanes(int newLaneCount)  {
            // if number of lanes is changed, 
            ArrayList <LaneRowColors> tempArrayList = new ArrayList<LaneRowColors>();

            for (int i=0; i< newLaneCount +1; i++)  {
                if(i < proposedColorArray.size())
                    tempArrayList.add(proposedColorArray.get(i).getCopy());
                else if (i<laneColorsArray.size())
                    tempArrayList.add(laneColorsArray.get(i).getCopy());
                else {
                    tempArrayList.add(new LaneRowColors(i,Color.BLACK,Color.WHITE));
                }
            }
            proposedColorArray.clear();
            proposedColorArray = tempArrayList;
            fireTableDataChanged();
        }
        
        public void acceptChange() {
            // Makes the proposed array the actual lane color array
            // then creates another copy for editing.
            laneColorsArray = proposedColorArray;
            copyToProposedArray();
            fireTableDataChanged();
        }
        
        public void cancelChange () {
            // Restores proposed array to a copy of lane color array
            copyToProposedArray();
            fireTableDataChanged();
        }

        private void copyToProposedArray()  {
            // Creates an independant copy which can be modified without
            // affecting the original lane color array.
            proposedColorArray = new ArrayList<LaneRowColors>();
            LaneRowColors tempRow;
            for (int i=0; i< laneColorsArray.size(); i++)  {
                tempRow = new LaneRowColors();
                tempRow.rowColors[0] = laneColorsArray.get(i).rowColors[0];
                tempRow.rowColors[1] = laneColorsArray.get(i).rowColors[1];
                tempRow.laneLabel = 
                        new JLabel(laneColorsArray.get(i).laneLabel.getText());
                tempRow.laneLabel.setForeground(tempRow.rowColors[0]);
                tempRow.laneLabel.setBackground(tempRow.rowColors[1]);
                
                proposedColorArray.add(tempRow);
            }
            fireTableDataChanged();
        }

        public int getRowCount() {
            return (proposedColorArray.size());
        }

        public int getColumnCount() {
            return 3;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            Object temp = null;
            switch(columnIndex)  {
                case 0:
                    temp = proposedColorArray.get(rowIndex).laneLabel;
                    break;
                case 1:
                    temp = proposedColorArray.get(rowIndex).rowColors[0];
                    break;
                case 2:
                    temp = proposedColorArray.get(rowIndex).rowColors[1];
            }
            return temp;
        }

        @Override
        public String getColumnName(int col) {
            return columnNames[col];
        }

        @Override
        public boolean isCellEditable(int row, int col) {
            boolean isEditable = true;
            if (col == 0) isEditable = false;
            return isEditable;
        }

        @Override
        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        @Override
        public void setValueAt(Object value, int row, int col) {
            switch (col) {
                case 0:
                    break;
                case 1:
                    proposedColorArray.get(row).rowColors[0] = (Color) value;
                    proposedColorArray.get(row).laneLabel.
                            setForeground((Color) value);
                    fireTableCellUpdated(row, 0);
                    break;
                case 2:
                    proposedColorArray.get(row).rowColors[1] = (Color) value;
                    proposedColorArray.get(row).laneLabel.
                            setBackground((Color) value);
                    fireTableCellUpdated(row, 0);
                    break;
            }
            fireTableCellUpdated(row, col);
        }
    }
    
    private class StartingHeatTableModel extends AbstractTableModel  {
        private String columnNames[];
        private int heatLineUp[][];
        private int proposedHeatLineUp[][];

        public StartingHeatTableModel()  {
            columnNames = new String[numberOfLanes+1];
            columnNames[0] = "Race";
            for (int i=1; i<numberOfLanes+1; i++)
                columnNames[i] = "Lane " + i;
            heatLineUp = new int[3][5];
            proposedHeatLineUp = new int [3][5];

            roundRobinLookUp();

            // copy to heatLineUp array
            for (int j=0; j< 3; j++)
                for (int i=0; i<5; i++)
                    heatLineUp[j][i] = proposedHeatLineUp[j][i];
        }

        public int[][] getPreliminaryHeatLineUp ()  {
            // [race][lane]
            int temp[][] = new int[heatLineUp.length-1][heatLineUp[0].length-1];
            for (int i=0; i<heatLineUp.length-1; i++)
                for (int j=1; j<heatLineUp[0].length; j++)
                    temp[i][j-1] = heatLineUp[i][j]-1;
            return temp;
        }

        public int[][] getFinalsHeatLineUp()  {
            int temp[][] = new int[1][heatLineUp[0].length-1];
            for (int j=1; j<heatLineUp[0].length; j++)
                temp[0][j-1] = heatLineUp[0][j]-1;
            return temp;
        }

        public void updateRowCount( int rpl)  {
            // update copies from the original and fills empty cells with 1

            int tempHeatLineUp[][] = new int [rpl+1][proposedHeatLineUp[0].length];
            for (int row = 0; row < rpl; row ++)  {  // copy all but last row
                for (int col = 0; col < tempHeatLineUp[0].length; col++)  {
                    if(row < proposedHeatLineUp.length-1)
                        tempHeatLineUp[row][col] = proposedHeatLineUp[row][col];
                    else
                        tempHeatLineUp[row][col] = 1;
                }
            }

            for (int col = 0; col < tempHeatLineUp[0].length; col++)  {
                tempHeatLineUp[tempHeatLineUp.length-1][col] =
                        proposedHeatLineUp[proposedHeatLineUp.length-1][col];
            }

            proposedHeatLineUp = tempHeatLineUp;
            fireTableDataChanged();
        }
        public void updateColumnCount (int newColumnCount)  {
            // update copies from the original and fills empty with 1

            int tempHeatLineUp[][] = new int [proposedHeatLineUp.length][newColumnCount+1];

            columnNames = new String[newColumnCount+1];
            columnNames[0] = "Race";
            for (int i=1; i<newColumnCount+1; i++)
                columnNames[i] = "Lane " + i;

            for (int row = 0; row < tempHeatLineUp.length; row ++)  {
                for (int col =0; col < newColumnCount+1; col++)  {
                    if (col < proposedHeatLineUp[0].length)
                        tempHeatLineUp[row][col] = proposedHeatLineUp[row][col];
                    else
                        tempHeatLineUp[row][col] = 1;
                }
            }
            proposedHeatLineUp = tempHeatLineUp;
            fireTableStructureChanged();
        }
        public void acceptChange()  {
            heatLineUp = proposedHeatLineUp;
            
            proposedHeatLineUp = new int[heatLineUp.length][heatLineUp[0].length];
            for (int j=0; j<heatLineUp.length; j++)
                for (int i=0; i<heatLineUp[0].length; i++)
                    proposedHeatLineUp[j][i] = heatLineUp[j][i];

            fireTableStructureChanged();
        }
        public void cancelChange()  {
            proposedHeatLineUp = new int[heatLineUp.length][heatLineUp[0].length];
            for (int j=0; j<heatLineUp.length; j++)
                for (int i=0; i<heatLineUp[0].length; i++)
                    proposedHeatLineUp[j][i] = heatLineUp[j][i];

            columnNames = new String[heatLineUp[0].length];
            columnNames[0] = "Race";
            for (int i=1; i<heatLineUp[0].length; i++)
                columnNames[i] = "Lane " + i;

            fireTableStructureChanged();
        }

        public int getRowCount() {
            return proposedHeatLineUp.length;
        }

        public int getColumnCount() {
            return proposedHeatLineUp[0].length;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            if (columnIndex == 0 && rowIndex == (proposedHeatLineUp.length-1))
                return "Finals";
            else
                return proposedHeatLineUp[rowIndex][columnIndex];
        }
        @Override
        public String getColumnName(int col) {
            return columnNames[col];
        }

        @Override
        public boolean isCellEditable(int row, int col) {
            boolean isEditable = false;
            if (editStartingHeatLineUp && col>0) isEditable = true;
            return isEditable;
        }
        @Override
        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        @Override
        public void setValueAt(Object value, int row, int col) {
            try  {
                if(((Integer) value) < 1)
                    throw new Exception("Invalid Number");
                proposedHeatLineUp[row][col] = (Integer) value;
            }
            catch (Exception err) {
                //do nothing, refreash will restore normal value
            }

            fireTableCellUpdated(row, col);
        }
        public void setPrelimiaryLineUps(int[][] newData) {
            if (newData !=null)  {
                proposedHeatLineUp = new int[newData.length+1][newData[0].length+1];
                columnNames = new String[proposedHeatLineUp[0].length];
                columnNames[0] = "Race";
                for (int i=1; i<proposedHeatLineUp[0].length; i++)
                columnNames[i] = "Lane " + i;
                roundRobinLookUp(); // sets initial values
                for (int row = 0; row < newData.length; row++)  {
                    proposedHeatLineUp[row][0] = row+1;
                    for (int col = 0; col < newData[0].length; col++)  {
                        proposedHeatLineUp[row][col+1] = newData[row][col];
                    }
                }
            }

            fireTableStructureChanged();
        }
        public void setAllLineUps(int[][] newData) {
            if (newData !=null)  {
                proposedHeatLineUp = new int[newData.length][newData[0].length+1];
                heatLineUp = new int[newData.length][newData[0].length+1];
                columnNames = new String[proposedHeatLineUp[0].length];
                columnNames[0] = "Race";
                for (int i=1; i<proposedHeatLineUp[0].length; i++)
                columnNames[i] = "Lane " + i;
                roundRobinLookUp(); // sets initial values
                for (int row = 0; row < newData.length; row++)  {
                    proposedHeatLineUp[row][0] = row+1;
                    heatLineUp[row][0] = row+1;
                    for (int col = 0; col < newData[0].length; col++)  {
                        proposedHeatLineUp[row][col+1] = newData[row][col];
                        heatLineUp[row][col+1] = newData[row][col];
                    }
                }
            }


            fireTableStructureChanged();
        }
        public int[][] saveAllLineUps()  {
            int temp[][] = new int[heatLineUp.length][heatLineUp[0].length];
            for (int row = 0; row < heatLineUp.length; row++)  {
                for (int col=0; col < heatLineUp[0].length; col++)  {
                    temp[row][col] = heatLineUp[row][col];
                }
            }
            return temp;
        }

        public void roundRobinLookUp()  {
            // Looks up the current rows and colums for the round robin and sets
            // the table parameters.
            // If not available, sets it up like a lane rotation.

            int oneRaceIndex = -1;
            int[][] selectedRace = null;
            
            // initial setup (lane rotation line up)
            for (int row = 0; row < proposedHeatLineUp.length; row++)  {
                proposedHeatLineUp[row][0] = row+1;
                for (int col = 1; col<proposedHeatLineUp[0].length; col++)  {
                    proposedHeatLineUp[row][col] = col;
                }
            }

            // set oneRaceIndex for finals and one race per lane type lineups
            if (proposedHeatLineUp[0].length == 13)  {
                oneRaceIndex = 8;
            }
            else if(proposedHeatLineUp[0].length >=4 && proposedHeatLineUp[0].length < 12) {
                oneRaceIndex = proposedHeatLineUp[0].length-4;
            }

            if(proposedHeatLineUp.length == 2 && oneRaceIndex !=-1)  {

                for (int row = 0; row < 2; row++)  {
                    proposedHeatLineUp[row][0] = row+1;
                    for (int col = 1; col<proposedHeatLineUp[0].length; col++)  {
                        proposedHeatLineUp[row][col] = oneRaceLineup[oneRaceIndex][col-1];
                    }
                }
            }

            else if(proposedHeatLineUp[0].length == 4 && oneRaceIndex !=-1)  {
                if(proposedHeatLineUp.length == 3)
                    selectedRace = LineUp3_2;
                else if(proposedHeatLineUp.length == 4)
                    selectedRace = LineUp3_3;
                else if(proposedHeatLineUp.length == 5)
                    selectedRace = LineUp3_4;
            }
            else if(proposedHeatLineUp[0].length == 5 && oneRaceIndex !=-1)  {
                if(proposedHeatLineUp.length == 3)
                    selectedRace = LineUp4_2;
                else if(proposedHeatLineUp.length == 4)
                    selectedRace = LineUp4_3;
                else if(proposedHeatLineUp.length == 5)
                    selectedRace = LineUp4_4;
            }
            else if(proposedHeatLineUp[0].length == 6 && oneRaceIndex !=-1)  {
                if(proposedHeatLineUp.length == 3)
                    selectedRace = LineUp5_2;
                else if(proposedHeatLineUp.length == 4)
                    selectedRace = LineUp5_3;
            }
            else if(proposedHeatLineUp[0].length == 7 && oneRaceIndex !=-1)  {
                if(proposedHeatLineUp.length == 3)
                    selectedRace = LineUp6_2;
            }
            
            if(selectedRace !=null && oneRaceIndex !=-1)  {
                // preliminary heat lineups
                for (int row = 0; row < proposedHeatLineUp.length-1; row++)  {
                    proposedHeatLineUp[row][0] = row+1;
                    for (int col = 1; col<proposedHeatLineUp[0].length; col++)  {
                        proposedHeatLineUp[row][col] = selectedRace[row][col-1];
                    }
                }

                // finals heat lineup
                for (int col = 1; col<proposedHeatLineUp[0].length; col++)  {
                    proposedHeatLineUp[proposedHeatLineUp.length-1][col] = oneRaceLineup[oneRaceIndex][col-1];
                }
            }
            fireTableDataChanged();
        }
    }

    private class OkButtonActionListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
//            System.out.println("Updating and closing");
            // set all values to current field values
            // Display Options
            laneColorTableModel.acceptChange();

            // regular Font
            int tempFontSize = raceLaneFont.getSize();
            try  {
                tempFontSize = Integer.
                        parseInt(raceFontSizeTextField.getText());
            }
            catch(Exception err)  {// failed parse, change to previous value
               tempFontSize = raceLaneFont.getSize();
               raceFontSizeTextField.setText(""+tempFontSize);
            }
            int tempFontStyle = Font.PLAIN;
            if(raceBoldCheckBox.isSelected())
                tempFontStyle = Font.BOLD;
            String tempFontName = (String) raceFontComboBox.getSelectedItem();

            raceLaneFont = new Font(tempFontName, tempFontStyle, tempFontSize);

            // Full Screen Font
            tempFontSize = raceFullScreenFont.getSize();
            try {
                tempFontSize = Integer.parseInt(raceFullScreenFontSizeTextField.getText());
            }
            catch (Exception err) { // failed parse, change to previous value
                tempFontSize = raceFullScreenFont.getSize();
                raceFullScreenFontSizeTextField.setText("" + tempFontSize);
            }
            tempFontStyle = Font.PLAIN;
            if(raceFullScreenBoldCheckBox.isSelected())
                tempFontStyle = Font.BOLD;
            raceFullScreenFont =
                    new Font(tempFontName, tempFontStyle, tempFontSize);

            // other Values
            textUseLaneColors = useLaneColorsCheckBox.isSelected();
            useCarColors = useCarColorsCheckBox.isSelected();
            try  {
                borderSize = Integer.
                        parseInt(laneBorderSizeTextField.getText());
            }
            catch (Exception err) { // failed parse, change to previous value
                laneBorderSizeTextField.setText("" + borderSize);
            }
            
            for(int i=0; i<laneItemSpacing.length; i++)
            {
                try  {
                    laneItemSpacing[i] = Integer.
                            parseInt(laneSpacingTextFields[i].getText());
                }  
                catch (Exception err) {// failed parse, change to previous value
                    laneSpacingTextFields[i].setText(""+laneItemSpacing[i]);
                }
            }

            // Race Options
            startingHeatTableModel.acceptChange();

            try {
                numberOfLanes = Integer.parseInt(numberOfLanesTextField.getText());
            }
            catch (Exception err)  {
                numberOfLanesTextField.setText(""+numberOfLanes);
            }
            raceType = raceTypeComboBox.getSelectedIndex();

            try  {
                racesPerLane = Integer.parseInt(racesPerLaneTextField.getText());
            }
            catch (Exception err)  {
                racesPerLaneTextField.setText(""+racesPerLane);
            }
            try  {
                numberOfFinalist = Integer.parseInt(numberOfFinalistTextField.getText());
                if (numberOfFinalist < numberOfLanes)  {
                    numberOfFinalist = numberOfLanes;
                    numberOfFinalistTextField.setText(""+numberOfFinalist);
                }
            }
            catch (Exception err)  {
                numberOfFinalistTextField.setText(""+numberOfFinalist);
            }
            placeCheckBoxes = displayPlaceCheckBox.isSelected();
            holdFinalsHeat = holdFinalsHeatCheckBox.isSelected();
            randomShuffleHeats = shuffleHeatCheckBox.isSelected();
            nameDisplayFormat = nameDisplayComboBox.getSelectedIndex();
            useLeaders = useLeadersCheckBox.isSelected();
            saveRaceData = saveRaceDataCheckBox.isSelected();

            proposedRacesPerLane = racesPerLane;
            proposedNumOfLanes = numberOfLanes;
            updated = true;
            setVisible(false);
        }
    }
    
    private class CancelButtonActionListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            // set all fields to values currently recored

            // Display Options
            laneColorTableModel.cancelChange();
            
            raceFontComboBox.setSelectedItem(raceLaneFont.getFamily());
            raceFontSizeTextField.setText(""+raceLaneFont.getSize());
            if(raceLaneFont.getStyle() == Font.BOLD)
                raceBoldCheckBox.setSelected(true);
            else raceBoldCheckBox.setSelected(false);
            raceFullScreenFontSizeTextField.
                    setText(""+raceFullScreenFont.getSize());
            if(raceFullScreenFont.getStyle() == Font.BOLD)
                raceFullScreenBoldCheckBox.setSelected(true);
            else raceFullScreenBoldCheckBox.setSelected(false);
            useLaneColorsCheckBox.setSelected(textUseLaneColors);
            useCarColorsCheckBox.setSelected(useCarColors);
            laneBorderSizeTextField.setText("" +borderSize);

            for(int i=0; i<laneItemSpacing.length; i++)
                laneSpacingTextFields[i].setText(""+laneItemSpacing[i]);
            
            // Race Options
            startingHeatTableModel.cancelChange();
            numberOfLanesTextField.setText("" +numberOfLanes);
            raceTypeComboBox.setSelectedIndex(raceType);
            racesPerLaneTextField.setText("" + racesPerLane);
            displayPlaceCheckBox.setSelected(placeCheckBoxes);
            numberOfFinalistTextField.setText("" + numberOfFinalist);
            holdFinalsHeatCheckBox.setSelected(holdFinalsHeat);
            shuffleHeatCheckBox.setSelected(randomShuffleHeats);
            nameDisplayComboBox.setSelectedIndex(nameDisplayFormat);
            useLeadersCheckBox.setSelected(useLeaders);
            saveRaceDataCheckBox.setSelected(saveRaceData);
            
            proposedRacesPerLane = racesPerLane;
            proposedNumOfLanes = numberOfLanes;

            updated = false;
            setVisible(false);
        }
    }
    private class numberOfLanesFocusListener implements FocusListener {

        public void focusGained(FocusEvent e) {
            // do nothing
        }

        public void focusLost(FocusEvent e) {
            // updates lane color table and starting heat table
            int tempInt;
            tempInt = proposedNumOfLanes;
            boolean updated = false;
            try {
                tempInt = Integer.parseInt(numberOfLanesTextField.getText());
                if(tempInt < 1) throw new Exception("Invalid Number");
                if (tempInt != proposedNumOfLanes) updated = true;
            }
            catch (Exception err)  {
                numberOfLanesTextField.setText("" + proposedNumOfLanes);
                tempInt = proposedNumOfLanes;
                updated = false;
            }
            if(updated)  {
                startingHeatTableModel.updateColumnCount(tempInt);
                laneColorTableModel.updateLanes(tempInt);
                proposedNumOfLanes = tempInt;
            }
        }
    }
    private class RacesPerLaneFocusListener implements FocusListener {

        public void focusGained(FocusEvent e) {
            // do nothing
        }

        public void focusLost(FocusEvent e) {
            // updates starting heat table
            int tempInt;
            tempInt = proposedRacesPerLane;
            boolean updated = false;
            try {
                tempInt = Integer.parseInt(racesPerLaneTextField.getText());
                if(tempInt < 1) throw new Exception("Invalid Number");
                if (tempInt != proposedRacesPerLane) updated = true;
            }
            catch (Exception err)  {
                racesPerLaneTextField.setText("" + proposedRacesPerLane);
                tempInt = proposedRacesPerLane;
                updated = false;
            }
            
            if (updated)  {
                startingHeatTableModel.updateRowCount(tempInt);
                proposedRacesPerLane = tempInt;
            }
        }
    }
    private class LoadDefaultActionListener implements ActionListener  {
        public void actionPerformed(ActionEvent e)  {
            startingHeatTableModel.roundRobinLookUp();
        }
    }


    private class RaceTypeActionListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            // shuffle types:
            //   0 - Chaotic Rotation (RR double shuffle)
            //   1 - Simple Chaotic Rotation
            //   2 - Lane rotation
            //   3 - Round Robin (Increment)
            //   4 - Timed Race Chaotic Rotation Lineup
            //   5 - Timed Race Simple Chaotic Rotation Lineup
            //   6 - Timed Race Round Robin Lineup
            switch (raceTypeComboBox.getSelectedIndex())  {
                case 0:
                    // Chaotic Rotation (RR double shuffle)
                    // set parameters
                    editStartingHeatLineUp = true;
                    timeField = false;
                    shuffleHeatCheckBox.setSelected(false);
                    displayPlaceCheckBox.setSelected(true);

                    // enable and dissable displays
//                    racesPerLaneTextField.setEnabled(true);
                    displayPlaceCheckBox.setEnabled(false);
//                    holdFinalsHeatCheckBox.setEnabled(true);
//                    numberOfFinalistTextField.setEnabled(true);
                    shuffleHeatCheckBox.setEnabled(false);
                    startingHeatTable.setEnabled(true);

                    break;
                case 1:
                    // Simple Chaotic Rotation
                    editStartingHeatLineUp = false;
                    timeField = false;
                    displayPlaceCheckBox.setSelected(true);
                    shuffleHeatCheckBox.setSelected(false);

                    // enable and dissable displays
//                    racesPerLaneTextField.setEnabled(false);
                    displayPlaceCheckBox.setEnabled(false);
//                    holdFinalsHeatCheckBox.setEnabled(false);
//                    numberOfFinalistTextField.setEnabled(false);
                    shuffleHeatCheckBox.setEnabled(false);
                    startingHeatTable.setEnabled(false);
                    break;
                case 2:
                    // Lane Rotation
                    editStartingHeatLineUp = false;
                    timeField = false;
                    displayPlaceCheckBox.setSelected(true);

                    // enable and dissable displays
//                    racesPerLaneTextField.setEnabled(true);
                    displayPlaceCheckBox.setEnabled(false);
//                    holdFinalsHeatCheckBox.setEnabled(true);
//                    numberOfFinalistTextField.setEnabled(true);
                    shuffleHeatCheckBox.setEnabled(true);
                    startingHeatTable.setEnabled(false);
                    break;
                case 3:
                    // Round Robin
                    editStartingHeatLineUp = true;
                    timeField = false;
                    displayPlaceCheckBox.setSelected(true);

                    // enable and dissable displays
//                    racesPerLaneTextField.setEnabled(true);
                    displayPlaceCheckBox.setEnabled(false);
//                    holdFinalsHeatCheckBox.setEnabled(true);
//                    numberOfFinalistTextField.setEnabled(true);
                    shuffleHeatCheckBox.setEnabled(true);
                    startingHeatTable.setEnabled(true);
                    break;
                case 4:
                    // Timed Race Chaotic Rotation Lineup
                    editStartingHeatLineUp = true;
                    timeField = true;
                    displayPlaceCheckBox.setSelected(false);
                    shuffleHeatCheckBox.setSelected(false);

                    // enable and dissable displays
//                    racesPerLaneTextField.setEnabled(true);
                    displayPlaceCheckBox.setEnabled(true);
//                    holdFinalsHeatCheckBox.setEnabled(true);
//                    numberOfFinalistTextField.setEnabled(true);
                    shuffleHeatCheckBox.setEnabled(false);
                    startingHeatTable.setEnabled(true);
                    break;
                case 5:
                    // Timed Race Simple Chaotic Rotation Lineup
                    editStartingHeatLineUp = false;
                    timeField = true;
                    displayPlaceCheckBox.setSelected(false);
                    shuffleHeatCheckBox.setSelected(false);

                    // enable and dissable displays
//                    racesPerLaneTextField.setEnabled(true);
                    displayPlaceCheckBox.setEnabled(true);
//                    holdFinalsHeatCheckBox.setEnabled(true);
//                    numberOfFinalistTextField.setEnabled(true);
                    shuffleHeatCheckBox.setEnabled(false);
                    startingHeatTable.setEnabled(false);
                    break;
                case 6:
                    // Timed Race, Round Robin Lineup
                    editStartingHeatLineUp = true;
                    timeField = true;
                    displayPlaceCheckBox.setSelected(false);

                    // enable and dissable displays
//                    racesPerLaneTextField.setEnabled(true);
                    displayPlaceCheckBox.setEnabled(true);
//                    holdFinalsHeatCheckBox.setEnabled(true);
//                    numberOfFinalistTextField.setEnabled(true);
                    shuffleHeatCheckBox.setEnabled(true);
                    startingHeatTable.setEnabled(true);
            }
        }
    }
    private class tabChangeListener implements ChangeListener  {

        public void stateChanged(ChangeEvent e) {
            int index = dialogTabbedPane.getSelectedIndex();
            int tempInt1, tempInt2;
            int[][] tempData = null;
            if (index == 2)  {
                tempInt1 = Integer.parseInt(numberOfLanesTextField.getText());
                tempInt2 = Integer.parseInt(racesPerLaneTextField.getText());
                roundRobinCalcPanel.setRace(tempInt1, tempInt2);
            }
            else if(index ==0 || index == 1) {
                tempData = roundRobinCalcPanel.getAvailableLineUp();
                if (tempData != null)  {
                    startingHeatTableModel.setPrelimiaryLineUps(tempData);
                    racesPerLaneTextField.setText("" + tempData.length);
                    numberOfLanesTextField.setText("" + tempData[0].length);
                    laneColorTableModel.updateLanes(tempData[0].length);
                    proposedNumOfLanes = tempData[0].length;
                    proposedRacesPerLane = tempData.length;
                }
            }
        }
    }

    public void windowOpened(WindowEvent e) {
    }

    public void windowClosing(WindowEvent e) {
         cancelButton.doClick();
    }

    public void windowClosed(WindowEvent e) {
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }

    public void windowActivated(WindowEvent e) {
    }

    public void windowDeactivated(WindowEvent e) {
    }
    private void setUpRaceArrays()  {
        oneRaceLineup = new int[9][];  // ragged array

        int[] t3 = {1,2,4};
        int[] t4 = {1,2,4,10};
        int[] t5 = {1,2,4,15,17};
        int[] t6 = {1,2,4,9,13,19};
        int[] t7 = {1,2,4,16,21,39,43};
        int[] t8 = {1,2,4,14,33,37,44,53};
        int[] t9 = {1,2,4,8,16,32,37,55,64};
        int[] t10 = {1,2,4,10,28,50,57,62,78,82};
        int[] t12 = {1,2,4,13,21,35,39,82,89,95,105,110};
        int[][] t3_2 = {{1,2,5}, {1,3,8}};
        int[][] t3_3 = {{1,2,5}, {1,3,10}, {1,6,12}};
        int[][] t3_4 = {{1,2,4}, {1,5,12}, {1,6,14}, {1,7,16}};
        int[][] t4_2 = {{1,2,4,10}, {1,5,12,17}};
        int[][] t4_3 = {{1,2,4,25}, {1,5,10,16}, {1,8,18,26}};
        int[][] t4_4 = {{1,2,4,9}, {1,5,19,30}, {1,7,22,34}, {1,10,20,33}};
        int[][] t5_2 = {{1,2,5,12,30}, {1,3,9,18,23}};
        int[][] t5_3 = {{1,2,4,14,35}, {1,5,10,24,46}, {1,7,18,25,33}};
        int[][] t6_2 = {{1,2,4,35,41,52}, {1,5,10,26,46,54}};

        oneRaceLineup[0] = t3;
        oneRaceLineup[1] = t4;
        oneRaceLineup[2] = t5;
        oneRaceLineup[3] = t6;
        oneRaceLineup[4] = t7;
        oneRaceLineup[5] = t8;
        oneRaceLineup[6] = t9;
        oneRaceLineup[7] = t10;
        oneRaceLineup[8] = t12;
        LineUp3_2 = t3_2;
        LineUp3_3 = t3_3;
        LineUp3_4 = t3_4;
        LineUp4_2 = t4_2;
        LineUp4_3 = t4_3;
        LineUp4_4 = t4_4;
        LineUp5_2 = t5_2;
        LineUp5_3 = t5_3;
        LineUp6_2 = t6_2;
    }
}
