package pwrace2;
import java.util.*;
import java.awt.Color;
import javax.swing.table.AbstractTableModel;
import javax.swing.*;

/**
 * This class will hold all the entries in an array and provide different
 * functions such as sort by car, name, or score.  Also it will provide
 * functions for generating the racing order and compiling the finals heat.
 * @author ghewitt
 */

public class PollArray extends AbstractTableModel {
    private String[] columnNames;
    private boolean isRacing;
    private boolean isTimedRace;
    private boolean includeLeaders;
    private boolean shuffleHeats;
    private int [][] roundRobinStart;
    private int numberOfLanes;
    private int raceType;
    private int racesPerLane;
    private Random myRand;

    private ArrayList <PollEntry> pollEntryArray;
    public final int PRELIMINARY = 0;
    public final int FINALS = 1;
    public PollEntry emptyLane;

    public PollArray()  {
        pollEntryArray = new ArrayList<PollEntry>();
        emptyLane = new PollEntry();
        String[] tNames = {"Car #", "Last Name", "First Name", " ",
                    "Number color", "Car Color", "Leader", "Score" };
        columnNames = tNames;
        isRacing = false;
        roundRobinStart = null;
        isTimedRace = false;
        includeLeaders = false;
        shuffleHeats = false;
        numberOfLanes = 4;
        raceType = 0;
        racesPerLane = 2;
        myRand = new Random();
    }

    public boolean isEmpty()  {
        return pollEntryArray.isEmpty();
    }

    public boolean isDuplicateCarNumber(int carNum) {
        boolean temp = false;
        if (pollEntryArray.isEmpty()) {
            temp = true;
        }
        else {
            for (int i =0; i<pollEntryArray.size(); i++)
                if (pollEntryArray.get(i).carNumber == carNum) temp = true;
        }
        return temp;
    }
    public void sortCarNum()  {
        boolean sorted = false;
        PollEntry tempPollEntry;
        if (pollEntryArray.isEmpty()) sorted = true;

        while (!sorted)
        {
            sorted = true;
            for (int i=0; i<pollEntryArray.size()-1; i++)
            {
                if (pollEntryArray.get(i).carNumber >
                        pollEntryArray.get(i+1).carNumber) {
                    sorted = false;
                    tempPollEntry = pollEntryArray.remove(i);
                    pollEntryArray.add(i+1,tempPollEntry);
                }
            }
        }

        fireTableDataChanged();
    }
    public void sortName() {
        boolean sorted = false;
        PollEntry tempPollEntry;
        String tempString1, tempString2;

        if (pollEntryArray.isEmpty()) sorted = true;

        while (!sorted)
        {
            sorted = true;
            for (int i=0; i<pollEntryArray.size()-1; i++)
            {
                tempString1 = pollEntryArray.get(i).lastName +
                        pollEntryArray.get(i).firstName;
                tempString2 = pollEntryArray.get(i+1).lastName +
                        pollEntryArray.get(i+1).firstName;

                if (tempString1.compareTo(tempString2) > 0) {
                    sorted = false;
                    tempPollEntry = pollEntryArray.remove(i);
                    pollEntryArray.add(i+1,tempPollEntry);
                }
            }
        }

        fireTableDataChanged();
    }
    public void sortScore() {
        boolean sorted = false;
        PollEntry tempPollEntry;
        if (pollEntryArray.isEmpty()) sorted = true;

        while (!sorted)
        {
            sorted = true;
            for (int i=0; i<pollEntryArray.size()-1; i++)
            {
                if(isTimedRace)  {
                    if (pollEntryArray.get(i).getAverageTime() >
                            pollEntryArray.get(i+1).getAverageTime()) {
                        sorted = false;
                        tempPollEntry = pollEntryArray.remove(i);
                        pollEntryArray.add(i+1,tempPollEntry);
                    }
                }
                else {
                    if (pollEntryArray.get(i).getScore() >
                            pollEntryArray.get(i+1).getScore()) {
                        sorted = false;
                        tempPollEntry = pollEntryArray.remove(i);
                        pollEntryArray.add(i+1,tempPollEntry);
                    }
                }
            }
        }

        fireTableDataChanged();
    }
    public String[] getList() {
        // This list is for poll entry mode, before the race begins
        sortCarNum();
        String[] tmpList = null;
        tmpList = new String[pollEntryArray.size()];

        for (int i=0; i<pollEntryArray.size();i++)  {
            tmpList[i] = pollEntryArray.get(i).getString();
        }

        return tmpList;
    }
    public String[] getListAll() {
        // This list is for Race mode, to save race data
        sortCarNum();
        String[] tmpList = null;
        tmpList = new String[pollEntryArray.size()];

        for (int i=0; i<pollEntryArray.size();i++)  {
            tmpList[i] = pollEntryArray.get(i).getAllString();
        }

        return tmpList;
    }
    public String[] getListForm() {
        // This list is for Race mode, to save race data
        sortCarNum();
        String[] tmpList = null;
        String[] tmpEntry = null;
        int sz = pollEntryArray.get(0).getFormString().length;
        tmpList = new String[pollEntryArray.size()*sz];

        for (int i=0; i<pollEntryArray.size();i++)  {
            tmpEntry = pollEntryArray.get(i).getFormString();
            for(int j=0; j< sz; j++)  {
                tmpList[sz*i+j] = tmpEntry[j];
            }
        }

        return tmpList;
    }

    public String[] getScoreList() {
        // This list will be used during the race in the entry box.

        String[] tmpList = null;
        int tmpSize = pollEntryArray.size() +  1;
        tmpList = new String[tmpSize];

        for (int i=0; i<pollEntryArray.size();i++)  {
            tmpList[i] = pollEntryArray.get(i).getRacingString();
        }
        tmpSize = pollEntryArray.size();
        tmpList[tmpSize]="";

        return tmpList;
    }
    public int getNumberOfMembers() {
        // This is for counting the members to generate the initial poll.

        int memberCount = 0;
        for (int i=0; i<pollEntryArray.size(); i++)
            if(!pollEntryArray.get(i).leader)
                memberCount++;

        return memberCount;
    }
    public int getPollSize() {
        return pollEntryArray.size();
    }

   public int getTotalEntries() {
    // do I need this one?
       
       return (pollEntryArray.size());
   }
    
    public PollEntry getEntryCopy(int carNum) {
        // This method will be used to get a copy of an entry for displaying.
        PollEntry temp;
        int t = findEntry(carNum);

        if (t==-1)  temp = null;
        else temp = pollEntryArray.get(t).getCopy();

        return temp;
    }

    public void addEntry(PollEntry nEntry) {
        pollEntryArray.add(nEntry);
    }
    
    public void replaceEntry (int oldCarNum, PollEntry rEntry) {
        pollEntryArray.set(findEntry(oldCarNum),rEntry);
    }

    // The delete Entry is used by the PollPannel to delete a specific entry.
    public void deleteEntry(int Index)  {
        pollEntryArray.remove(Index);
    }

    public void setRacing()  {
        isRacing = true;
        fireTableStructureChanged();
    }
    
    public void clearRace()  {
        for(int i=0; i<pollEntryArray.size(); i++)
            pollEntryArray.get(i).clearRace();
        isRacing = false;
        fireTableStructureChanged();
    }

    public ArrayList <PollEntry> getWinners()  {
        ArrayList <PollEntry> tPoll = new ArrayList<PollEntry>();
        sortScore();
        for (int i=0; i<3; i++) {
            tPoll.add(pollEntryArray.get(i).getCopy());
        }
        return tPoll;
    }
    public void setTimedRace(boolean tr)  {
        isTimedRace = tr;
    }

    public void setIncludeLeaders(boolean il)  {
        includeLeaders = il;
    }

    public void setShuffleHeats(boolean sh) {
        shuffleHeats = sh;
    }

    public void setNumberOfLanes(int lns)  {
        numberOfLanes = lns;
    }

    public void setRaceType(int rt)  {
        raceType = rt;
    }

    public void setRacesPerLane(int rpl)  {
        racesPerLane = rpl;
    }

    public void setRoundRobinStart(int[][] rRobin)  {
        roundRobinStart = new int[rRobin.length][rRobin[0].length];
        for (int i=0; i<rRobin.length; i++)
            for (int j=0; j<rRobin[0].length; j++)
                roundRobinStart[i][j] = rRobin[i][j];
    }

    public int[][] createPreliminaryLineUp()  {
        int retPoll[][];
        ArrayList <PollEntry> tArray = new ArrayList<PollEntry>();

        for (int index = 0; index < pollEntryArray.size(); index++)
            pollEntryArray.get(index).setLanes(racesPerLane, numberOfLanes);

        // Create preliminary poll Entries (include leaders or not)
        for (int i = 0; i< pollEntryArray.size(); i++)
            if(!pollEntryArray.get(i).leader || includeLeaders)
                tArray.add(pollEntryArray.get(i));

        retPoll = createLineup(tArray);
        return retPoll;
    }

    public int[][] createFinalsLineUp(JFrame parent, int numOfFinalists, boolean useCarColors)  {
        int retPoll[][];
        int index = 0;
        boolean moreFinalists = true;
        ArrayList <PollEntry> tArray = new ArrayList<PollEntry>();

        // Exclude Leaders from finals
        for (int i=0; i< pollEntryArray.size(); i++)
            if(pollEntryArray.get(i).leader)
                pollEntryArray.get(i).setNonFinalist();
        sortScore();

        while (moreFinalists)  {
            tArray.add(pollEntryArray.get(index));
            pollEntryArray.get(index).clearRace();
            pollEntryArray.get(index).setLanes(racesPerLane, numberOfLanes);
            index ++;
            if (index >= pollEntryArray.size())
                moreFinalists = false;
            else if(!(index < numOfFinalists ||
                    pollEntryArray.get(index).getScore() == 
                    pollEntryArray.get(index-1).getScore()))  {
                moreFinalists = false;
            }
        }
        
        while (index < pollEntryArray.size())  {
            pollEntryArray.get(index).setNonFinalist();
            index ++;
        }
        FinalistsDialog d = new FinalistsDialog(parent, tArray, useCarColors);
        retPoll = createLineup(tArray);
        return retPoll;
    }

    private int[][] createLineup(ArrayList <PollEntry> pollArray) {

        if(pollArray.size() < numberOfLanes)  {
            JOptionPane.showMessageDialog(null, "Not Enough Racers", "Error",
                    JOptionPane.ERROR_MESSAGE);
            return null;
        }

        // This is a generic race lineup
        sortCarNum();
        int i, j, k, t;

        if (raceType == 4 || raceType == 5)
            isTimedRace = true;
        else
            isTimedRace = false;
        
        ArrayList <PollEntry> tempArray = new ArrayList<PollEntry>() ;

        for (j=0; j<racesPerLane; j++)
            for (i=0; i < pollArray.size();i++)
                tempArray.add(pollArray.get(i));

        t = tempArray.size();

        int [][] tPoll = new int [t][numberOfLanes];
        
        for(i=0; i<tPoll.length; i++)
            for(j=0; j<tPoll[i].length; j++)
                tPoll[i][j] = -1;
        // shuffle types:
        //   0 - Chaotic Rotation (RR double shuffle)
        //   1 - Simple Chaotic Rotation
        //   2 - Lane rotation
        //   3 - Round Robin (Increment)
        //   4 - Timed Race Chaotic Rotation Lineup
        //   5 - Timed Race Simple Chaotic Rotation Lineup
        //   6 - Timed Race Round Robin Lineup

        if(raceType == 1 || raceType == 5)  {
            // Simple Caotic Rotation,
            // completly Random for place and timed events
            i=0;

            // Place cars randomly in all lanes and heats
            for(j=0; j<numberOfLanes; j++)  {
                i=0;
                while(!tempArray.isEmpty())  {
                    t = myRand.nextInt(tempArray.size());
                    tPoll[i][j] = tempArray.get(t).carNumber;
                    tempArray.remove(t);
                    i++;
                }

                // refill tempArray
                for (k=0; k<racesPerLane; k++)
                    for (i=0; i < pollArray.size();i++)
                        tempArray.add(pollArray.get(i));
            }

            // Verify there is no duplicate entries
            randomSwapDuplicates(tPoll);

            // check for racers in successive heats
            boolean badLineUp = true;
            for (int tries = 0; tries < 10 && badLineUp; tries++)  {
                badLineUp = false;
                for (i=0; i<tPoll.length-1; i++)  { // increment all heats
                    for (j=0; j<tPoll[0].length; j++)  { // reference heat
                        for(k = 0; k<tPoll[0].length; k++)  { // check heat
                            if(tPoll[i][j] == tPoll[i+1][k])  {
                                badLineUp = true;
                                swapHeats(tPoll, (i+1));
                            }
                        }
                    }
                }
            }
        }
        
        if(raceType == 2)  {  // Lane Rotation
            int tIndex;
            sortCarNum();
            for (int laneIndex =0; laneIndex < numberOfLanes; laneIndex++)  {
                tIndex = laneIndex;
                for (int index = 0; index <  tempArray.size(); index++)  {
                    tPoll[index][laneIndex] = tempArray.get(tIndex).carNumber;
                    tIndex++;
                    if (tIndex >= tempArray.size())
                        tIndex = tIndex - tempArray.size();
                }
            }

            // Shuffle heats
            if(shuffleHeats)  {
                int[][] tPoll2 = new int [tPoll.length][tPoll[0].length];
                ArrayList <Integer> indexList = new ArrayList<Integer>();
                ArrayList <Integer> shuffleIndex = new ArrayList<Integer>();
                for (int index = 0; index < tPoll.length; index++)  {
                    indexList.add(new Integer(index));
                }
                while (!indexList.isEmpty())  {
                    shuffleIndex.add(indexList.remove(myRand.nextInt(indexList.size())));
                }
                for (int index = 0; index < tPoll.length; index++)
                    tPoll2[index] = tPoll[shuffleIndex.get(index).intValue()];
                tPoll = tPoll2;

                // check for racers in successive heats
                boolean badLineUp = true;
                for (int tries = 0; tries < 10 && badLineUp; tries++)  {
                    badLineUp = false;
                    for (i=0; i<tPoll.length-1; i++)  { // incremet all heats
                        for (j=0; j<tPoll[0].length; j++)  { // reference heat
                            for(k = 0; k<tPoll[0].length; k++)  { // check heat
                                if(tPoll[i][j] == tPoll[i+1][k])  {
                                    badLineUp = true;
                                    swapHeats(tPoll, (i+1));
                                }
                            }
                        }
                    }
                }
            }
        }

        if(raceType == 0 || raceType == 3 || raceType == 4 || raceType == 6)  {
            // Round Robin (Increment)  and RR double shuffle type
            // both place and timed events
            tempArray.clear();
            sortCarNum();
            int tIndex;
            ArrayList<PollEntry> tArray2 = new ArrayList<PollEntry>();
            
                
            for (i=0; i < pollArray.size();i++)
                tempArray.add(pollArray.get(i));

            if(raceType == 0 || raceType == 4)  {  // shuffle line up
                while (!tempArray.isEmpty())  {
                    t = myRand.nextInt(tempArray.size());
                    tArray2.add(tempArray.remove(t));
                }
                tempArray = tArray2;
                shuffleHeats = true;  // ensure shuffle heats
            }

            t = tempArray.size();

            for (int race = 0; race < racesPerLane; race++)  {
                for (int laneIndex =0; laneIndex < numberOfLanes; laneIndex++) {
                    tIndex = roundRobinStart[race][laneIndex];
                    while(tIndex >= t)  { // verify tIndex inside array size
                        tIndex = tIndex - t;
                    }

                    for (int index = 0; index <  t; index++)  {
                        tPoll[index+ race*t][laneIndex] =
                                tempArray.get(tIndex).carNumber;
                        tIndex++;
                        if (tIndex >= t)
                            tIndex = tIndex - t;
                    }
                }
            }
            randomSwapDuplicates(tPoll);
            
            // Shuffle heats
            if(shuffleHeats)  {
                int[][] tPoll2 = new int [tPoll.length][tPoll[0].length];
                ArrayList <Integer> indexList = new ArrayList<Integer>();
                ArrayList <Integer> shuffleIndex = new ArrayList<Integer>();
                for (int index = 0; index < tPoll.length; index++)  {
                    indexList.add(new Integer(index));
                }
                while (!indexList.isEmpty())  {
                    shuffleIndex.add(indexList.remove(myRand.nextInt(indexList.size())));
                }
                for (int index = 0; index < tPoll.length; index++)
                    tPoll2[index] = tPoll[shuffleIndex.get(index).intValue()];
                tPoll = tPoll2;

                // check for racers in successive heats
                boolean badLineUp = true;
                for (int tries = 0; tries < 10 && badLineUp; tries++)  {
                    badLineUp = false;
                    for (i=0; i<tPoll.length-1; i++)  { // incremet all heats
                        for (j=0; j<tPoll[0].length; j++)  { // reference heat
                            for(k = 0; k<tPoll[0].length; k++)  { // check heat
                                if(tPoll[i][j] == tPoll[i+1][k])  {
                                    badLineUp = true;
                                    swapHeats(tPoll, (i+1));
                                }
                            }
                        }
                    }
                }
            }
        }

        // set heats in entries
        if(tPoll !=null)
            for (int lane = 0; lane < numberOfLanes; lane ++)  {
                for (int heat=0; heat<tPoll.length; heat++)  {
                    pollEntryArray.get(findEntry(tPoll[heat][lane])).addHeat(heat, lane);
                }
            }
        return tPoll;
    }
    private void randomSwapDuplicates(int [][] tPoll)  {
        // Verify there is no duplicate entries
        boolean duplicate;
        int desHeat, tCar;
        for (int curLane=0; curLane<numberOfLanes-1; curLane++)  {
            for (int curHeat=0; curHeat<tPoll.length; curHeat++)  {
                for (int ckLane=curLane+1; ckLane<numberOfLanes; ckLane++) {
                    if(tPoll[curHeat][curLane] == tPoll[curHeat][ckLane]) {

                        // Swap Cars
                        // cycle until no duplicate is found.
                        duplicate = true;
                        desHeat = 0;
                        while (duplicate)  {
                            duplicate = false;
                            desHeat = myRand.nextInt(tPoll.length);

                            // check current car is not in destination heat
                            for (int dhLane=0; dhLane<numberOfLanes; dhLane++)
                                if (tPoll[curHeat][ckLane] == tPoll[desHeat][dhLane])
                                    duplicate = true;

                            // check target car is not in current lane
                            for (int chLane=0; chLane<numberOfLanes; chLane++)
                                if (tPoll[desHeat][ckLane] == tPoll[curHeat][chLane])
                                    duplicate = true;
                        }
                        // Swapping cars
                        tCar = tPoll[desHeat][ckLane];
                        tPoll[desHeat][ckLane] = tPoll[curHeat][ckLane];
                        tPoll[curHeat][ckLane] = tCar;
                    }
                }
            }
        }
    }

    private void swapHeats(int[][] tPoll, int heat)  {
        // This function is to find a spot for the selected heat where the
        // previous and next heats do not contain a racer in the selected heat.
        // It then verifies target heat does not have a racer in selected
        // heats previous heat.
        int tHeat[] = new int[tPoll[0].length];
        boolean nvSwap = true; // not valid swap
        for (int i=0; i< tPoll.length && nvSwap; i++)  {
            nvSwap = false;
            for (int j=0; j< tPoll[0].length; j++)  {
                for (int k=0; k< tPoll[0].length; k++)  {
                    // check target heat with selected heat -1
                    if(tPoll[i][j] == tPoll[heat-1][k])  {
                        nvSwap = true;
                    }
                    // check selected heat with target heat
                    if(i>0 && tPoll[heat][j] == tPoll[i-1][k])  {
                        nvSwap = true;
                    }
                    if(i<tPoll.length-1 && tPoll[heat][j] == tPoll[i+1][k])  {
                        nvSwap = true;
                    }
                }
            }
            if (!nvSwap)  {  // valid swap, switching heats
                tHeat = tPoll[i];
                tPoll[i] = tPoll[heat];
                tPoll[heat] = tHeat;
            }
        }
    }
    
    public void recordRaceResults(int heat, int[] cars, int[] results, double[] rTime) {
    //    if (heat >= 0 && heat < pollEntryArray.size())
            for (int i=0; i<cars.length; i++)  {
                pollEntryArray.get(findEntry(cars[i])).setPlace(heat, results[i]);
                pollEntryArray.get(findEntry(cars[i])).setTime(heat, rTime[i]);
        }
    }

    private int getCarPlace(int carNum, int heat)  { // not used
        int tempPlace = -1;
        if (heat >=0 && heat < pollEntryArray.size())// error, heat can be > size()
            tempPlace = pollEntryArray.get(findEntry(carNum)).getPlace(heat);
        return tempPlace;
    }

    private int findEntry(int carNum) {
        // This is an internal method to find the index to a particular car
        // The reference could be a privacy leak so the method is private.
        
        int carIndex=-1;
        for (int i=0; i< pollEntryArray.size(); i++){
            if (pollEntryArray.get(i).carNumber == carNum)
                carIndex = i;
        }
        return carIndex;
    }
    
    public void clearEntries() {
        pollEntryArray.clear();
    }

    /*
     * The following codes define the methods used to create the table
     * for editing the Poll
     */
    public int getRowCount(){
        return pollEntryArray.size();
    }
    public int getColumnCount() {
        int colCount = 7;
        if (isRacing) colCount = 8;
        return colCount;
    }
    public Object getValueAt(int row, int column)  {
        Object tempObject = null;
        JLabel tempLabel;

        if(row > pollEntryArray.size()) column = 10; // return null
        PollEntry temp = pollEntryArray.get(row).getCopy();
        switch (column) {
            case 0:
                tempObject = temp.carNumber;
                break;
            case 1:
                tempObject = temp.lastName;
                break;
            case 2:
                tempObject = temp.firstName;
                break;
            case 3:
                tempLabel = new JLabel("" +temp.carNumber);
                tempLabel.setForeground(temp.foregroundColor);
                tempLabel.setBackground(temp.backgroundColor);
                tempObject = tempLabel;
                break;
            case 4:
                tempObject = temp.foregroundColor;
                break;
            case 5:
                tempObject = temp.backgroundColor;
                break;
            case 6:
                tempObject = temp.leader;
                break;
            case 7:
                if(isTimedRace)
                    tempObject = temp.getAverageTime();
                else
                    tempObject = temp.getScore();
        }
        return tempObject;

    }
    @Override
    public String getColumnName(int col) {
        return columnNames[col];
    }

    @Override
    public boolean isCellEditable(int row, int col) {
        boolean isEditable = true;
        if (col == 3 || col == 7) isEditable = false;
        return isEditable;
    }

    @Override
    public Class getColumnClass(int c) {
        return getValueAt(0, c).getClass();
    }

    @Override
    public void setValueAt(Object value, int row, int col) {
        int tempInt = -1;
        boolean duplicate = false;

        switch (col) {
            case 0:
                // need to scan to ensure no duplicate copies
                tempInt = (Integer) value;
                if (tempInt < 0) {
                    JOptionPane.showMessageDialog(null, "Car number must be " +
                            "positive");
                    duplicate = true;
                }
                for (int i=0; i<pollEntryArray.size(); i++)
                    if(i != row)
                        if(tempInt == pollEntryArray.get(i).carNumber)  {
                            duplicate = true;
                            JOptionPane.showMessageDialog(null, "Number " +
                                    "already used");
                        }
                if(!duplicate) pollEntryArray.get(row).carNumber = tempInt;
                fireTableCellUpdated(row, 3);
                break;
            case 1:
                pollEntryArray.get(row).lastName = (String)value;
                break;
            case 2:
                pollEntryArray.get(row).firstName = (String)value;
                break;
            case 3:
                break;
            case 4:
                pollEntryArray.get(row).foregroundColor = (Color) value;
                fireTableCellUpdated(row, 3);
                break;
            case 5:
                pollEntryArray.get(row).backgroundColor = (Color) value;
                fireTableCellUpdated(row, 3);
                break;
            case 6:
                pollEntryArray.get(row).leader = (Boolean) value;
        }
        fireTableCellUpdated(row, col);
    }
}
