package pwrace2;
import java.awt.Color;

/**
 * This class is very similar to the previous PollEntry class.  It holds the
 * data of each car and owner and keeps track of the score.  Since it is mostly
 * a data structure type class, the atributes are public.
 *
 * @author ghewitt
 */
public class PollEntry {

    public String lastName, firstName;
    public int carNumber;
    private int heats[];
    private int places[];
    private double times[];
    private int lanes;
    public Color foregroundColor;
    public Color backgroundColor;
    public boolean leader;

    public PollEntry()
    {
        lastName = "";
        firstName = "";
        carNumber = -1;
        foregroundColor = Color.BLACK;
        backgroundColor = Color.WHITE;
        leader = false;
        lanes = 0;
        heats = null;
        places = null;
        times = null;
    }

    public PollEntry(String textLine)
    {
        int r=0, g=0, b=0;
        lanes = 0;
        heats = null;
        places = null;
        times = null;
        // Convert Text line from file to entries
        String data[] = textLine.split(",");

        for(int i=0; i<data.length;i++)
            data[i] = data[i].trim();


        // If Error, fill with default and error text.
        if(data.length ==10)  {
            // used if entering a new racer
            try  {
                carNumber = Integer.parseInt(data[0]);
                lastName = data[1];
                firstName = data[2];
                r = Integer.parseInt(data[3]);
                g = Integer.parseInt(data[4]);
                b = Integer.parseInt(data[5]);
                foregroundColor = new Color(r,g,b);
                r = Integer.parseInt(data[6]);
                g = Integer.parseInt(data[7]);
                b = Integer.parseInt(data[8]);
                backgroundColor = new Color(r,g,b);
                leader = data[9].equals("Leader");
            }
            catch(Exception err)  {
                lastName = "";
                firstName = "";
                carNumber = -1;
                foregroundColor = Color.BLACK;
                backgroundColor = Color.WHITE;
                leader = false;
            }
        }
        else if(data.length > 10)  {
            // used if loading a race
            int temp = 0;
            try  {
                carNumber = Integer.parseInt(data[0]);
                lastName = data[1];
                firstName = data[2];
                r = Integer.parseInt(data[3]);
                g = Integer.parseInt(data[4]);
                b = Integer.parseInt(data[5]);
                foregroundColor = new Color(r,g,b);
                r = Integer.parseInt(data[6]);
                g = Integer.parseInt(data[7]);
                b = Integer.parseInt(data[8]);
                backgroundColor = new Color(r,g,b);
                leader = data[9].equals("Leader");

                // race information
                lanes = Integer.parseInt(data[10]);
                temp = (data.length - 11)/3;
                if ((data.length-11)%3 !=0)
                    throw new Exception("Invalid Racer Entry");
                heats = new int[temp];
                places = new int[temp];
                times = new double[temp];
                for (int i=0; i<temp; i++)  {
                    heats[i] = Integer.parseInt(data[i*3+11]);
                    places[i] = Integer.parseInt(data[i*3+12]);
                    times[i] = Double.parseDouble(data[i*3+13]);
                }
            }
            catch(Exception err)  {
                lastName = "Error";
                firstName = err.getMessage();
                carNumber = -1;
                foregroundColor = Color.BLACK;
                backgroundColor = Color.WHITE;
                leader = false;
            }
        }
        else  {
            lastName = "";
            firstName = "";
            carNumber = -1;
            foregroundColor = Color.BLACK;
            backgroundColor = Color.WHITE;
            leader = false;
        }

    }

    public String getString()
    {
        String racerString, temp;

        temp = ", " + foregroundColor.getRed();
        temp += ", " + foregroundColor.getGreen();
        temp += ", " + foregroundColor.getBlue();
        temp += ", " + backgroundColor.getRed();
        temp += ", " + backgroundColor.getGreen();
        temp += ", " + backgroundColor.getBlue();
        temp += ", ";

        if(leader)
            temp += "Leader";
        else
            temp += "Member";

        racerString = carNumber+", "+lastName+", "+firstName+temp;

        return racerString;
    }

    public String getRacingString()
    {
        String racerString, temp;
        if(leader)
            temp = "Leader";
        else
            temp = "Member";
        temp += "   Score: " + getScore();
        racerString = carNumber+", "+lastName+", "+firstName+", "+temp;

        return racerString;
    }

    public String getAllString()
    {
        /**
         * This routine will be used for storing all data to a file.
         * Most likely used for saving a race.
         */

        String racerString, temp;

        temp = " ," + foregroundColor.getRed();
        temp += "," + foregroundColor.getGreen();
        temp += "," + foregroundColor.getBlue();
        temp += "," + backgroundColor.getRed();
        temp += "," + backgroundColor.getGreen();
        temp += "," + backgroundColor.getBlue();

        if(leader)
            temp += ",Leader";
        else
            temp += ",Member";

        temp = temp + "," + lanes;

        for(int i=0; i<heats.length;i++)
        {
            temp = temp+","+heats[i]+","+places[i]+","+times[i];
        }
        
        racerString = carNumber+", "+lastName+", "+firstName+temp;

        return racerString;
    }
    public String[] getFormString()  {
        String[] formString = new String[1+heats.length/lanes];
        String temp = "";
        formString[0] = null;
        formString[1] = null;
        if(leader) temp = "Leader";
        formString[0] = String.format("%3d, %1s, %1s, %1s",
                carNumber, lastName, firstName, temp);
        if(heats !=null)  {
            for(int j = 0; j < formString.length-1; j++)  {
                formString[j+1] = String.format("   [%3d],______", (heats[j*lanes]+1));
                for(int i=1; i<lanes; i++)  {
                    formString[j+1] = formString[j+1] +
                            String.format(", [%3d],______", (heats[j*lanes+i]+1));
                }
            }
        }
        return formString;
    }

    public void setLanes(int races, int lns)
    {
        lanes = lns;
        heats = new int[races*lanes];
        places = new int[races*lanes];
        times = new double[races*lanes];

        // setting heat to -1 scores to 1 more than lanes, and max time.
        for(int i=0; i<heats.length; i++)  {
            heats[i] = -1;
            places[i] = lanes +1;
            times[i] = 999;
        }
    }

    public void clearRace()  {
        lanes = 0;
        heats = null;
        places = null;
        times = null;
    }

    public void addHeat(int heat, int lane)
    {
        boolean usedHeat = true;
        int translatedIndex = lane;

        while(usedHeat && translatedIndex<heats.length)
        {
            if(heats[translatedIndex] < 0)
            {
                usedHeat = false;
                heats[translatedIndex] = heat;
            }
            else
                translatedIndex += lanes;
        }
    }

    public void setPlace(int heat, int pl)
    {
        // search for heat
        for(int i=0; i<heats.length; i++)
        {
            if(heats[i] == heat)
                places[i] = pl;
        }
    }
    public void setTime(int heat, double tm)
    {
        // search for heat
        for(int i=0; i<heats.length; i++)
        {
            if(heats[i] == heat)
                times[i] = tm;
        }
    }

    public int getPlace(int heat)
    {
        int sc = lanes+1;

        for(int i=0; i<heats.length; i++)
        {
            if(heats[i] == heat)
                sc = places[i];
        }

        return sc;
    }

    public double getTime(int heat)
    {
        double tm = 999;

        for(int i=0; i<heats.length; i++)
        {
            if(heats[i] == heat)
                tm = times[i];
        }

        return tm;
    }

    public int getScore()
    {
        // using place as score, like Golf.
        int totalScore = 0;
        int temp;
        if (places != null)  {
            temp = places.length;

            for(int i=0; i<temp; i++)
                totalScore += places[i];

        }
        return totalScore;
    }
    public double getAverageTime()  {
        double average = 0;
        if (times != null)  {
            int temp = times.length;

            for(int i=0; i<temp; i++)
                average += times[i];
            
            average = average/temp;
        }
        
        return average;
    }

    public PollEntry getCopy()
    {
        PollEntry rtnTemp = new PollEntry();
        rtnTemp.carNumber = carNumber;
        rtnTemp.lastName = new String(lastName);
        rtnTemp.firstName = new String(firstName);
        rtnTemp.foregroundColor = new Color(foregroundColor.getRGB());
        rtnTemp.backgroundColor = new Color(backgroundColor.getRGB());
        rtnTemp.leader = leader;
        if (heats!= null)  {
            rtnTemp.lanes = lanes;

            rtnTemp.heats = new int[heats.length];
            for (int i=0; i< heats.length; i++)
                rtnTemp.heats[i] = heats[i];

            rtnTemp.places = new int[places.length];
            for (int i=0; i< places.length; i++)
                rtnTemp.places[i] = places[i];

            rtnTemp.times = new double[times.length];
            for (int i=0; i< times.length; i++)
                rtnTemp.times[i] = times[i];
        }
        else  {
            rtnTemp.heats = null;
            rtnTemp.places = null;
            rtnTemp.times = null;
            rtnTemp.lanes = 0;
        }

        return rtnTemp;
    }

    public void setNonFinalist()
    {
        if(places.length > 0)
            places[0] += 1000;
        if(times.length > 0)
            times[0] += 1000;
    }
}
