package pwrace2;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
 *
 * @author ghewitt
 */
public class PollPanel extends JPanel {
    private JLabel carNumLabel;
    private JLabel lastNameLabel;
    private JLabel firstNameLabel;
    private JTextField carNumTextField;
    private JTextField lastNameTextField;
    private JTextField firstNameTextField;
    private JTextField exampleNumTextField;
    private JCheckBox leaderCheckBox;
    private JButton color1Button;
    private JButton color2Button;
    private JButton addButton;
    private JButton deleteButton;
    private JPanel topPanel;
    private JPanel bottomPanel;
    private JScrollPane topSP;

    private JTable pollTable;
    private JScrollPane pollTablePane;

    private PollArray pollArray;

    public PollPanel(PollArray pArray)  {
        pollArray = pArray;

        // Create items for panels
        carNumLabel = new JLabel("Car #:");
        lastNameLabel = new JLabel("Last Name:");
        firstNameLabel = new JLabel("First Name:");
        carNumTextField = new JTextField(2);
        carNumTextField.addFocusListener(new CarNumFocusListener());
        exampleNumTextField = new JTextField(2);
        exampleNumTextField.setEditable(false);
        exampleNumTextField.setOpaque(true);
        exampleNumTextField.setFont(carNumTextField.getFont().
                deriveFont(Font.BOLD));
        exampleNumTextField.setHorizontalAlignment(JTextField.CENTER);
        exampleNumTextField.setBackground(Color.WHITE);
        exampleNumTextField.setForeground(Color.BLACK);
        lastNameTextField = new JTextField(14);
        firstNameTextField = new JTextField(14);
        color1Button = new JButton("#");
        color1Button.setBackground(Color.BLACK);
        color1Button.setForeground(Color.WHITE);
        color1Button.addActionListener(new Color1ButtonListener());
        color2Button = new JButton("Car");
        color2Button.setBackground(Color.WHITE);
        color2Button.setForeground(Color.BLACK);
        color2Button.addActionListener(new Color2ButtonListener());
        leaderCheckBox = new JCheckBox("Leader");
        addButton = new JButton("Add");
        addButton.addActionListener(new AddButtonListener());
        deleteButton = new JButton("Delete");
        deleteButton.addActionListener(new DeleteButtonListener());
        
        // Create top panel
        topPanel = new JPanel();
        topPanel.setPreferredSize(new Dimension(850, 50));
        topPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        topPanel.add(carNumLabel);
        topPanel.add(carNumTextField);
        topPanel.add(lastNameLabel);
        topPanel.add(lastNameTextField);
        topPanel.add(firstNameLabel);
        topPanel.add(firstNameTextField);
        topPanel.add(exampleNumTextField);
        topPanel.add(color1Button);
        topPanel.add(color2Button);
        topPanel.add(leaderCheckBox);
        topPanel.add(addButton);
        topSP = new JScrollPane(topPanel);


        // Create middle panel

        pollTable = new JTable(pollArray);
        pollTablePane = new JScrollPane(pollTable);

        pollTable.setDefaultRenderer(Color.class, new ColorRenderer(true));
        pollTable.setDefaultEditor(Color.class, new ColorEditor());
        pollTable.setDefaultRenderer(JLabel.class, new CarNumberRenderer(true));
        pollTable.setAutoCreateRowSorter(true);
        pollTable.setColumnSelectionAllowed(false);
        
        int[]columnWidths = { 10, 220, 220, 10, 50, 50, 40};
        for (int i = 0; i < 7; i++) {
            pollTable.getColumnModel().getColumn(i).
                    setPreferredWidth(columnWidths[i]);
        }


        // Create bottom panel
        bottomPanel = new JPanel();
        bottomPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        bottomPanel.add(deleteButton);

        // add to main panel;
        setLayout(new BorderLayout());
        add(topSP, BorderLayout.NORTH);
        add(pollTablePane, BorderLayout.CENTER);
        add(bottomPanel, BorderLayout.SOUTH);


    }

    public void setRaceStarted(boolean raceStarted) {
        addButton.setEnabled(!raceStarted);
        deleteButton.setEnabled(!raceStarted);
        color1Button.setEnabled(!raceStarted);
        color2Button.setEnabled(!raceStarted);
        carNumTextField.setEnabled(!raceStarted);
        lastNameTextField.setEnabled(!raceStarted);
        firstNameTextField.setEnabled(!raceStarted);
        leaderCheckBox.setEnabled(!raceStarted);
        pollTable.setEnabled(!raceStarted);
        int[]columnWidths = { 10, 220, 220, 10, 50, 50, 40, 60};
        int cols = 7;
        if(raceStarted) cols = 8;
        for (int i = 0; i < cols; i++) {
            pollTable.getColumnModel().getColumn(i).
                    setPreferredWidth(columnWidths[i]);
        }

    }

    private class Color1ButtonListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            Color selectedColor;
            selectedColor = JColorChooser.showDialog(null, "Select a number " +
                    "color.", color1Button.getBackground());
            color1Button.setBackground(selectedColor);
            color2Button.setForeground(selectedColor);
            exampleNumTextField.setForeground(selectedColor);

        }
    }

    private class Color2ButtonListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            Color selectedColor;
            selectedColor = JColorChooser.showDialog(null, "Select a car " +
                    "color.", color2Button.getBackground());
            color2Button.setBackground(selectedColor);
            color1Button.setForeground(selectedColor);
            exampleNumTextField.setBackground(selectedColor);
        }
    }

    private class CarNumFocusListener implements FocusListener {

        public void focusGained(FocusEvent e) {
            // Not used
        }

        public void focusLost(FocusEvent e) {
            exampleNumTextField.setText(carNumTextField.getText());
        }
    }

    private class AddButtonListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            // adds the current name to the array

            PollEntry temp = new PollEntry();
            try
            {
                temp.carNumber = Integer.parseInt(carNumTextField.getText());

                if(lastNameTextField.getText().contains(","))
                    throw new IllegalArgumentException(
                            "Last name contains a ','");
                if(firstNameTextField.getText().contains(","))
                    throw new IllegalArgumentException(
                            "First name contains a ','");

                temp.firstName = firstNameTextField.getText();
                temp.lastName = lastNameTextField.getText();
                temp.foregroundColor = color1Button.getBackground();
                temp.backgroundColor = color2Button.getBackground();
                temp.leader = leaderCheckBox.isSelected();


                boolean exists = false;
                if(!pollArray.isEmpty())
                    exists = pollArray.isDuplicateCarNumber(temp.carNumber);

                // search for dubpicate car numbers
                if(!exists)
                    pollArray.addEntry(temp);
                else JOptionPane.showMessageDialog(null, "Car Number Already" +
                        "Used", "Error", JOptionPane.ERROR_MESSAGE);

            }
            catch (Exception err)
            {
                JOptionPane.showMessageDialog(null, err.getMessage(), "Error",
                        JOptionPane.ERROR_MESSAGE);
            }
            carNumTextField.setText("");
            firstNameTextField.setText("");
            lastNameTextField.setText("");
            exampleNumTextField.setText("");
            exampleNumTextField.setForeground(Color.BLACK);
            exampleNumTextField.setBackground(Color.WHITE);
            color1Button.setBackground(Color.BLACK);
            color1Button.setForeground(Color.WHITE);
            color2Button.setBackground(Color.WHITE);
            color2Button.setForeground(Color.BLACK);
            leaderCheckBox.setSelected(false);

            carNumTextField.requestFocusInWindow();
            pollArray.fireTableDataChanged();
        }
    }

    private class DeleteButtonListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            int[] deleteIndecies;
            int tempIndex;
            boolean sorted = false;
            deleteIndecies = pollTable.getSelectedRows();

            // ensure indecies are sorted
            if (deleteIndecies.length <=1) sorted = true;

            // convert view index to model index
            for (int i=0; i<deleteIndecies.length; i++)
                deleteIndecies[i] = pollTable.
                        convertRowIndexToModel(deleteIndecies[i]);

            while(!sorted)  {
                sorted = true;
                for (int i=0; i<deleteIndecies.length-1; i++)
                    if (deleteIndecies[i] > deleteIndecies[i+1]) {
                        tempIndex = deleteIndecies[i];
                        deleteIndecies[i] = deleteIndecies[i+1];
                        deleteIndecies[i+1] = tempIndex;
                        sorted = false;
                    }
            }

            // deleting
            for(int i=deleteIndecies.length-1; i>=0; i--)
                pollArray.deleteEntry(deleteIndecies[i]);
            pollArray.fireTableDataChanged();
        }
    }
}
