package pwrace2;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 *
 * @author ghewitt
 */
public class RacePanel extends JPanel {
    
    private PollArray pollArray;
    private int numberOfLanes;
    private int nameDisplay;
    private int currentRaceHeat;  // -1 = no heat selected
    private int raceEvent; // 0 = Preliminary, 1 = Finals, -1 = None
    private int raceLineUp [][];  // [heat] [lane], stores car number
    private int[] laneItemSpacing;
    private int laneHeight;
    private Font windowFont;
    private boolean useCarNumColors;
    private boolean useLaneColorsForTextFields;
    private boolean showPlaceCheckBoxes;
    private boolean showTimeTextField;
    private String placeStrings[];

    private JLabel laneHeaderLabel;
    private JLabel carNumHeaderLabel;
    private JLabel driverHeaderLabel;
    private JLabel placeLabels[];
    private JLabel timeHeaderLabel;
    private JPanel headerPanel;
    private JPanel headerPlacePanel;
    private JPanel buttonPanel;
    private JPanel lanePanels[];
    private JPanel lanePlacePanels[]; // holds check boxes
    private JLabel laneNumberLabel[];
    private JTextField carNumberTextFields[];
    private JTextField driverTextFields[];
    private JCheckBox placeCheckBoxes[][];
    private JButton previousHeatButton;
    private JButton nextHeatButton;
    private JTextField heatTypeAndNumTextField;
    private JLabel onDeckLabel;
    private JTextField onDeckTextFields[];
    private JPanel onDeckPanel;
    private JTextField timeTextFields[];
    private JScrollPane raceSP;
    private JPanel JP;
    
    private JButton nextEventButton;

    public RacePanel (PollArray pArray)  {
        pollArray = pArray;
        JP = new JPanel();
        raceSP = new JScrollPane(JP);
        setLayout(new GridLayout(1,1));
        JP.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        GridBagConstraints c2 = new GridBagConstraints();

        // Setting default values. for initial configuration.
        // Same as values in RaceOptions Dialog.  
        numberOfLanes = 4;
//        racesPerLane = 2; // may move to race options dialog
        raceEvent = -1;
        currentRaceHeat = -1;
        raceLineUp = null;
        nameDisplay = 0;
        useCarNumColors = true;
        placeStrings = new String[12];
        placeStrings[0] = "1st";
        placeStrings[1] = "2nd";
        placeStrings[2] = "3rd";
        for (int i=3; i<12; i++)
            placeStrings[i] = (i+1) + "th";
        showTimeTextField = false;
        showPlaceCheckBoxes = true;
        useLaneColorsForTextFields = false;
        
        windowFont = new Font("Dialog", Font.BOLD, 18);

        laneItemSpacing = new int[6];
        laneItemSpacing[0] = 50;
        laneItemSpacing[1] = 50;
        laneItemSpacing[2] = 400;
        laneItemSpacing[3] = 100;
        laneItemSpacing[4] = 200;
        laneItemSpacing[5] = 40;
        laneHeight = laneItemSpacing[5];

        // Display Panel;
        lanePlacePanels = new JPanel[numberOfLanes];

        setBackground(Color.WHITE);
        setForeground(Color.BLACK);
        JP.setBackground(Color.WHITE);
        JP.setForeground(Color.BLACK);
        laneHeaderLabel = new JLabel("Lane");
        laneHeaderLabel.setFont(windowFont);
        laneHeaderLabel.setForeground((getForeground()));
        laneHeaderLabel.setPreferredSize(new Dimension(laneItemSpacing[0],laneHeight));
        carNumHeaderLabel = new JLabel("Car");
        carNumHeaderLabel.setFont(windowFont);
        carNumHeaderLabel.setForeground((getForeground()));
        carNumHeaderLabel.setPreferredSize(new Dimension(laneItemSpacing[1],laneHeight));
        driverHeaderLabel = new JLabel("Driver");
        driverHeaderLabel.setFont(windowFont);
        driverHeaderLabel.setForeground((getForeground()));
        driverHeaderLabel.setPreferredSize(new Dimension(laneItemSpacing[2],laneHeight));
        timeHeaderLabel = new JLabel("Time");
        timeHeaderLabel.setFont(windowFont);
        timeHeaderLabel.setForeground((getForeground()));
        timeHeaderLabel.setPreferredSize(new Dimension(1, laneHeight));
        placeLabels = new JLabel[numberOfLanes];
        placeLabels[0] = new JLabel(placeStrings[0], SwingConstants.CENTER);
        placeLabels[1] = new JLabel(placeStrings[1], SwingConstants.CENTER);
        placeLabels[2] = new JLabel(placeStrings[2], SwingConstants.CENTER);
        placeLabels[3] = new JLabel(placeStrings[3], SwingConstants.CENTER);

        headerPlacePanel = new JPanel();
        headerPlacePanel.setBackground(getBackground());
        headerPlacePanel.setPreferredSize(
                new Dimension(laneItemSpacing[4], laneHeight));
        headerPlacePanel.setLayout(new GridLayout(1,numberOfLanes));

        for (int i=0; i<numberOfLanes; i++)  {
            placeLabels[i].setFont(windowFont);
            placeLabels[i].setForeground(getForeground());
            headerPlacePanel.add(placeLabels[i]);
        }
        
        headerPanel = new JPanel();
        headerPanel.setBackground(getBackground());
        headerPanel.setLayout((new GridBagLayout()));
        headerPanel.setBorder(BorderFactory.createEmptyBorder(10,10,0,10));

        c.anchor = GridBagConstraints.LINE_START;
        c.gridx = 0;
        c.gridy = 0;
        headerPanel.add(laneHeaderLabel, c);
        c.gridx++;
        headerPanel.add(carNumHeaderLabel, c);
        c.gridx++;
        headerPanel.add(driverHeaderLabel, c);
        c.gridx++;
        headerPanel.add(timeHeaderLabel, c);
        c.gridx++;
        headerPanel.add(headerPlacePanel,c);

        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.LINE_START;

        JP.add(headerPanel,c);

        // Lane Panels
        lanePanels = new JPanel[numberOfLanes];
        lanePlacePanels = new JPanel[numberOfLanes];
        laneNumberLabel = new JLabel[numberOfLanes];
        carNumberTextFields = new JTextField[numberOfLanes];
        driverTextFields = new JTextField[numberOfLanes];
        timeTextFields = new JTextField[numberOfLanes];
        placeCheckBoxes = new JCheckBox[numberOfLanes][numberOfLanes];

//        int driverTextFieldWidth = 20;
        for(int i=0; i<numberOfLanes; i++)  {
            c2.gridx=0;
            c2.gridy=0;
            c2.weightx=0;
            c2.weighty=0;
            lanePanels[i] = new JPanel();
            lanePanels[i].setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
            lanePanels[i].setLayout(new GridBagLayout());
            laneNumberLabel[i] = new JLabel(" "+(i+1)+".");
            if(i>1) laneNumberLabel[i].setForeground(Color.WHITE);
            laneNumberLabel[i].setFont(windowFont);
            laneNumberLabel[i].setPreferredSize(new Dimension(laneItemSpacing[0],laneHeight));

            carNumberTextFields[i] = new JTextField();
            carNumberTextFields[i].setFont(windowFont);
            carNumberTextFields[i].setBackground(getBackground());
            carNumberTextFields[i].setForeground(getForeground());
            carNumberTextFields[i].setHorizontalAlignment(JTextField.CENTER);
            carNumberTextFields[i].setEditable(false);
            carNumberTextFields[i].setPreferredSize(new Dimension(laneItemSpacing[1],laneHeight));
            carNumberTextFields[i].setFocusable(false);

            driverTextFields[i] = new JTextField();
            driverTextFields[i].setFont(windowFont);
            driverTextFields[i].setBackground(getBackground());
            driverTextFields[i].setForeground(getForeground());
            driverTextFields[i].setEditable(false);
            driverTextFields[i].setPreferredSize(new Dimension(laneItemSpacing[2],laneHeight));
            driverTextFields[i].setFocusable(false);

            timeTextFields[i] = new JTextField();
            timeTextFields[i].setFont(windowFont);
            timeTextFields[i].setBackground(getBackground());
            timeTextFields[i].setForeground(getForeground());
            timeTextFields[i].setEditable(false);
            timeTextFields[i].setPreferredSize(new Dimension(1,laneHeight));
            timeTextFields[i].addFocusListener(new TimeTextFieldFocusListener());
            
            lanePlacePanels[i] = new JPanel();
            lanePlacePanels[i].setOpaque(false);
            lanePlacePanels[i].setLayout(new GridLayout(1,numberOfLanes));
            lanePlacePanels[i].setPreferredSize(new Dimension(laneItemSpacing[4],laneHeight));

            for(int j=0; j<numberOfLanes; j++)  {
                placeCheckBoxes[i][j] = new JCheckBox();
                placeCheckBoxes[i][j].setBackground(getBackground());
                placeCheckBoxes[i][j].setForeground(getForeground());
                placeCheckBoxes[i][j].addActionListener(new PlaceCheckBoxListener());
                placeCheckBoxes[i][j].setHorizontalAlignment(SwingConstants.CENTER);
                placeCheckBoxes[i][j].setEnabled(false);
                placeCheckBoxes[i][j].setFocusable(false);
                lanePlacePanels[i].add(placeCheckBoxes[i][j]);
            }


            lanePanels[i].add(laneNumberLabel[i], c2);
            c2.gridx++;
            lanePanels[i].add(carNumberTextFields[i], c2);
            c2.gridx++;
            lanePanels[i].add(driverTextFields[i], c2);
            c2.gridx++;
            lanePanels[i].add(timeTextFields[i]);
            c2.gridx++;
            lanePanels[i].add(lanePlacePanels[i], c2);

            c.gridx = 0;
            c.gridy = i+1;
            JP.add(lanePanels[i],c);
        }

        lanePanels[0].setBackground(Color.YELLOW);
        lanePanels[1].setBackground(Color.GREEN);
        lanePanels[2].setBackground(Color.BLUE);
        lanePanels[2].setForeground(Color.WHITE);
        lanePanels[3].setBackground(Color.RED);
        lanePanels[3].setForeground(Color.WHITE);

        // adding bottom row which will take up the slack
        buttonPanel = new JPanel();
        buttonPanel.setBackground(getBackground());
        buttonPanel.setFont(windowFont);
        previousHeatButton = new JButton("<");
        previousHeatButton.addActionListener(new PreviousHeatButtonListener());
        previousHeatButton.setFont(windowFont);
        previousHeatButton.setEnabled(false);
        previousHeatButton.setForeground(getForeground());
        previousHeatButton.setBackground(getBackground());
        previousHeatButton.setFocusable(false);
        nextHeatButton = new JButton(">");
        nextHeatButton.addActionListener(new NextHeatButtonListener());
        nextHeatButton.setFont(windowFont);
        nextHeatButton.setEnabled(false);
        nextHeatButton.setForeground(getForeground());
        nextHeatButton.setBackground(getBackground());
        heatTypeAndNumTextField = new JTextField(15);
        heatTypeAndNumTextField.setText("N/A");
        heatTypeAndNumTextField.setEditable(false);
        heatTypeAndNumTextField.setFont(windowFont);
        heatTypeAndNumTextField.setForeground(getForeground());
        heatTypeAndNumTextField.setBackground(getBackground());
        heatTypeAndNumTextField.setFocusable(false);

        onDeckLabel = new JLabel("     On Deck:");
        onDeckLabel.setBackground(getBackground());
        onDeckLabel.setForeground(getForeground());
        onDeckLabel.setFont(windowFont);
        onDeckPanel = new JPanel();

        onDeckTextFields = new JTextField[numberOfLanes];

        Dimension tempDim = new Dimension(laneItemSpacing[1],laneHeight);
        for (int i=0; i<numberOfLanes; i++)  {
             onDeckTextFields[i] = new JTextField();
             onDeckTextFields[i].setFont(windowFont);
             onDeckTextFields[i].setPreferredSize(tempDim);
             onDeckTextFields[i].setHorizontalAlignment(JTextField.CENTER);
             onDeckTextFields[i].setEditable(false);
             onDeckTextFields[i].setForeground(getForeground());
             onDeckTextFields[i].setBackground(getBackground());
             onDeckTextFields[i].setFocusable(false);
        }
        buttonPanel.add(previousHeatButton);
        buttonPanel.add(heatTypeAndNumTextField);
        buttonPanel.add(nextHeatButton);

        c.gridy++;
        JP.add(buttonPanel,c);

        onDeckPanel = new JPanel();
        onDeckPanel.setBackground(getBackground());
        onDeckPanel.setForeground(getForeground());
        onDeckPanel.add(onDeckLabel);
        
        for (int i=0; i<numberOfLanes; i++)
            onDeckPanel.add(onDeckTextFields[i]);

        c.gridy++;
        c.weighty = 1;
        c.anchor = GridBagConstraints.FIRST_LINE_END;
        JP.add(onDeckPanel, c);
        add(raceSP);

    }
    
    
    // this method updated the panel if options changed.  
    public void regenerate()  {
        headerPanel.setBackground(getBackground());
        buttonPanel.setBackground(getBackground());
        nextHeatButton.setBackground(getBackground());
        nextHeatButton.setForeground(getForeground());
        nextHeatButton.setFont(windowFont);
        previousHeatButton.setBackground(getBackground());
        previousHeatButton.setForeground(getForeground());
        previousHeatButton.setFont(windowFont);
        heatTypeAndNumTextField.setBackground(getBackground());
        heatTypeAndNumTextField.setForeground(getForeground());
        heatTypeAndNumTextField.setFont(windowFont);
        onDeckLabel.setForeground(getForeground());
        onDeckLabel.setFont(windowFont);

        Dimension tempDim = new Dimension(laneItemSpacing[1],laneHeight);
        PollEntry tempEntry;
        for (int i=0; i<numberOfLanes; i++)  {
            onDeckTextFields[i].setFont(windowFont);
            onDeckTextFields[i].setPreferredSize(tempDim);
            if(useCarNumColors && (raceLineUp !=null) && (currentRaceHeat+1 < raceLineUp.length))  {
                tempEntry = pollArray.getEntryCopy(raceLineUp[currentRaceHeat+1][i]);
                onDeckTextFields[i].setBackground(tempEntry.backgroundColor);
                onDeckTextFields[i].setForeground(tempEntry.foregroundColor);
            }
            else {
                onDeckTextFields[i].setBackground(getBackground());
                onDeckTextFields[i].setForeground(getForeground());
            }
        }

        tempDim = new Dimension(laneItemSpacing[0],laneHeight);
        laneHeaderLabel.setForeground(getForeground());
        laneHeaderLabel.setFont(windowFont);
        laneHeaderLabel.setPreferredSize(tempDim);

        tempDim = new Dimension(laneItemSpacing[1],laneHeight);
        carNumHeaderLabel.setFont(windowFont);
        carNumHeaderLabel.setPreferredSize(tempDim);
        carNumHeaderLabel.setForeground(getForeground());

        tempDim = new Dimension(laneItemSpacing[2],laneHeight);
        driverHeaderLabel.setFont(windowFont);
        driverHeaderLabel.setPreferredSize(tempDim);
        driverHeaderLabel.setForeground(getForeground());

        if (showTimeTextField)
            tempDim = new Dimension(laneItemSpacing[3], laneHeight);
        else tempDim = new Dimension(1, laneHeight);
        timeHeaderLabel.setFont(windowFont);
        timeHeaderLabel.setPreferredSize(tempDim);
        timeHeaderLabel.setForeground(getForeground());

        if (showPlaceCheckBoxes)
            tempDim = new Dimension(laneItemSpacing[4],laneHeight);
        else tempDim = new Dimension (1, laneHeight);
        headerPlacePanel.setPreferredSize(tempDim);
        headerPlacePanel.setBackground(getBackground());

        for(int i=0; i<numberOfLanes; i++)  {
            placeLabels[i].setForeground(getForeground());
            placeLabels[i].setFont(windowFont);

            tempDim = new Dimension(laneItemSpacing[0],laneHeight);
            laneNumberLabel[i].setPreferredSize(tempDim);
            laneNumberLabel[i].setFont(windowFont);
            laneNumberLabel[i].setBackground(lanePanels[i].getBackground());
            laneNumberLabel[i].setForeground(lanePanels[i].getForeground());

            tempDim = new Dimension(laneItemSpacing[1],laneHeight);
            carNumberTextFields[i].setPreferredSize(tempDim);
            carNumberTextFields[i].setFont(windowFont);
            if(useCarNumColors)  { 
                // need to add code to get car colorsn or add another function
                // to refresh lane lane contents.
                carNumberTextFields[i].setBackground(getBackground());
                carNumberTextFields[i].setForeground(getForeground());
            }
            else  {
                if (useLaneColorsForTextFields)  {
                    carNumberTextFields[i].setBackground(lanePanels[i].getBackground());
                    carNumberTextFields[i].setForeground(lanePanels[i].getForeground());
                }
                else {  // use background colors
                    carNumberTextFields[i].setBackground(getBackground());
                    carNumberTextFields[i].setForeground(getForeground());
                }
            }

            tempDim = new Dimension(laneItemSpacing[2],laneHeight);
            driverTextFields[i].setFont(windowFont);
            if(useLaneColorsForTextFields)  {
                driverTextFields[i].setBackground(lanePanels[i].getBackground());
                driverTextFields[i].setForeground(lanePanels[i].getForeground());
            }
            else {
                driverTextFields[i].setBackground(getBackground());
                driverTextFields[i].setForeground(getForeground());
            }
            driverTextFields[i].setPreferredSize(tempDim);

            if(showTimeTextField)
                tempDim = new Dimension(laneItemSpacing[3], laneHeight);
            else
                tempDim = new Dimension(1,laneHeight);
            timeTextFields[i].setFont(windowFont);
            if(useLaneColorsForTextFields)  {
                timeTextFields[i].setForeground(lanePanels[i].getForeground());
                timeTextFields[i].setBackground(lanePanels[i].getBackground());
            }
            else  {
                timeTextFields[i].setForeground(getForeground());
                timeTextFields[i].setBackground(getBackground());
            }
            timeTextFields[i].setPreferredSize(tempDim);
            
            if(showPlaceCheckBoxes)
                tempDim = new Dimension(laneItemSpacing[4],laneHeight);
            else tempDim = new Dimension(1,laneHeight);
            lanePlacePanels[i].setFont(windowFont);
            lanePlacePanels[i].setBackground(lanePanels[i].getBackground());
            lanePlacePanels[i].setForeground(lanePanels[i].getForeground());
            lanePlacePanels[i].setPreferredSize(tempDim);

            for (int j=0; j<numberOfLanes; j++)  {
                placeCheckBoxes[i][j].setBackground(getBackground());
                placeCheckBoxes[i][j].setForeground(getForeground());
            }
        }
        validate();
    }

    public void setRacePanelFont(Font rFont)  {
        windowFont = rFont.deriveFont(rFont.getStyle());
    }

    public void setNumberOfLanes(int lanes)  {
        // need to rebuild each panel, then redisplay entire race panel
        JPanel tempLanePanels[];
        JPanel tempLanePlacePanels[];
        JLabel tempPlaceLabels[];
        JLabel tempLaneNumLabels[];
        JTextField tempCarNumTextFields[];
        JTextField tempDriverTextFields[];
        JTextField tempTimeTextFields[];
        JTextField tempOnDeckTextFields[];
        JCheckBox tempPlaceCheckBoxes[][];

        GridBagConstraints c = new GridBagConstraints();
        Dimension tempDim;

        if (lanes > 0)  {
            numberOfLanes = lanes;
            JP.removeAll();
            
            // ****************
            // ** Header Panel
            // ****************
            
            headerPlacePanel.removeAll();
            headerPlacePanel.setLayout(new GridLayout(1,numberOfLanes));
            tempPlaceLabels = new JLabel[numberOfLanes];
            for (int i=0; i<numberOfLanes; i++)  {
                if (i < placeLabels.length)
                    tempPlaceLabels[i] = placeLabels[i];
                else  {
                    if (numberOfLanes < 13)
                        tempPlaceLabels[i] = new JLabel(placeStrings[i], SwingConstants.CENTER);
                    else tempPlaceLabels[i] = new JLabel((i+1)+"", SwingConstants.CENTER);

                    tempPlaceLabels[i].setFont(windowFont);
                    tempPlaceLabels[i].setBackground(getBackground());
                }
                headerPlacePanel.add(tempPlaceLabels[i]);
            }
            placeLabels = tempPlaceLabels;
            
            // ***************
            // **  Lane Panel
            // ***************

            // Lane Number Label
            tempDim = new Dimension(laneItemSpacing[0],laneHeight);
            tempLaneNumLabels = new JLabel[numberOfLanes];
            for (int i=0; i<numberOfLanes; i++)  {
                if (i<laneNumberLabel.length)
                    tempLaneNumLabels[i] = laneNumberLabel[i];
                else  {
                    tempLaneNumLabels[i] = new JLabel(" "+(i+1)+".");
                    tempLaneNumLabels[i].setForeground(Color.BLACK);
                    tempLaneNumLabels[i].setFont(windowFont);
                    tempLaneNumLabels[i].setPreferredSize(tempDim);
                }
            }
            laneNumberLabel = tempLaneNumLabels;

            // Car Number Text Fields
            tempDim = new Dimension(laneItemSpacing[1],laneHeight);
            tempCarNumTextFields = new JTextField[numberOfLanes];
            for (int i=0; i< numberOfLanes; i++)  {
                if (i< carNumberTextFields.length)
                    tempCarNumTextFields[i] = carNumberTextFields[i];
                else  {
                    tempCarNumTextFields[i] = new JTextField();
                    tempCarNumTextFields[i].setFont(windowFont);
                    tempCarNumTextFields[i].setBackground(getBackground());
                    tempCarNumTextFields[i].setForeground(getForeground());
                    tempCarNumTextFields[i].setHorizontalAlignment(JTextField.CENTER);
                    tempCarNumTextFields[i].setEditable(false);
                    tempCarNumTextFields[i].setPreferredSize(tempDim);
                }
            }
            carNumberTextFields = tempCarNumTextFields;

            // Driver Text Fields
            tempDim = new Dimension(laneItemSpacing[2],laneHeight);
            tempDriverTextFields = new JTextField[numberOfLanes];
            for (int i=0; i<numberOfLanes; i++){
                if (i<driverTextFields.length)
                    tempDriverTextFields[i] = driverTextFields[i];
                else  {
                    tempDriverTextFields[i] = new JTextField();
                    tempDriverTextFields[i].setFont(windowFont);
                    tempDriverTextFields[i].setEditable(false);
                    tempDriverTextFields[i].setPreferredSize(tempDim);
                }
            }
            driverTextFields = tempDriverTextFields;
            
            // Time Text Fields
            tempDim = new Dimension(laneItemSpacing[3],laneHeight);
            tempTimeTextFields = new JTextField[numberOfLanes];
            for (int i=0; i< numberOfLanes; i++)  {
                if (i<timeTextFields.length)
                    tempTimeTextFields[i] = timeTextFields[i];
                else  {
                    tempTimeTextFields[i] = new JTextField();
                    tempTimeTextFields[i].setFont(windowFont);
                    tempTimeTextFields[i].setBackground(getBackground());
                    tempTimeTextFields[i].setEditable(false);
                    if(showTimeTextField)
                        tempDim = new Dimension(1, laneHeight);
                }
            }
            timeTextFields = tempTimeTextFields;

            // create checkBoxes
            tempPlaceCheckBoxes = new JCheckBox[numberOfLanes][numberOfLanes];
            for (int i=0; i< numberOfLanes; i++)  {
                for (int j=0; j< numberOfLanes; j++)  {
                    if (i<placeCheckBoxes.length && j<placeCheckBoxes[0].length)
                        tempPlaceCheckBoxes[i][j] = placeCheckBoxes[i][j];
                    else  {
                        tempPlaceCheckBoxes[i][j] = new JCheckBox();
                        tempPlaceCheckBoxes[i][j].setBackground(getBackground());
                        tempPlaceCheckBoxes[i][j].setForeground(getForeground());
                        tempPlaceCheckBoxes[i][j].addActionListener(new PlaceCheckBoxListener());
                        tempPlaceCheckBoxes[i][j].setHorizontalAlignment(SwingConstants.CENTER);
                        tempPlaceCheckBoxes[i][j].setEnabled(false);
                    }
                }
            }
            placeCheckBoxes = tempPlaceCheckBoxes;

            // Check box Panel
            tempLanePlacePanels = new JPanel[numberOfLanes];
            tempDim = new Dimension(laneItemSpacing[4],laneHeight);
            for(int i=0; i<numberOfLanes; i++)  {
                if (i<lanePlacePanels.length)  {
                    tempLanePlacePanels[i] = lanePlacePanels[i];
                    tempLanePlacePanels[i].removeAll();
                    tempLanePlacePanels[i].setLayout(new GridLayout(1, numberOfLanes));
                }
                else  {
                    tempLanePlacePanels[i] = new JPanel();
                    tempLanePlacePanels[i].setLayout(new GridLayout(1, numberOfLanes));
                    tempLanePlacePanels[i].setOpaque(false);
                    tempLanePlacePanels[i].setPreferredSize(tempDim);
                }
                for (int j=0; j<numberOfLanes; j++)
                    tempLanePlacePanels[i].add(tempPlaceCheckBoxes[i][j]);
            }
            lanePlacePanels = tempLanePlacePanels;
            
            tempLanePanels = new JPanel[numberOfLanes];

            for (int i=0; i<numberOfLanes; i++)  {
                c.gridx = 0;
                c.gridy = 0;
                c.anchor = GridBagConstraints.LINE_START;
                c.weightx = 0;
                c.weighty = 0;

                if (i<lanePanels.length)
                    tempLanePanels[i] = lanePanels[i];
                else  {  // build a new panel
                    tempLanePanels[i] = new JPanel();
                    tempLanePanels[i].setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
                    tempLanePanels[i].setLayout(new GridBagLayout());
                    tempLanePanels[i].add(laneNumberLabel[i], c);
                    c.gridx ++;
                    tempLanePanels[i].add(carNumberTextFields[i], c);
                    c.gridx ++;
                    tempLanePanels[i].add(driverTextFields[i], c);
                    c.gridx++;
                    tempLanePanels[i].add(timeTextFields[i]);
                    c.gridx++;
                    tempLanePanels[i].add(lanePlacePanels[i], c);
                    tempLanePanels[i].setBackground(Color.WHITE);
                    tempLanePanels[i].setForeground(Color.BLACK);
                }
            }
            lanePanels = tempLanePanels;

            // ****************
            // ** Button Panel
            // ****************
            buttonPanel.removeAll();
            buttonPanel.add(previousHeatButton);
            buttonPanel.add(heatTypeAndNumTextField);
            buttonPanel.add(nextHeatButton);
            
            onDeckPanel.add(onDeckLabel);
            tempOnDeckTextFields = new JTextField[numberOfLanes];
            tempDim = new Dimension(laneItemSpacing[1],laneHeight);
            for (int i=0; i<numberOfLanes; i++)  {
                if (i<onDeckTextFields.length)
                    tempOnDeckTextFields[i] = onDeckTextFields[i];
                else  {
                     tempOnDeckTextFields[i] = new JTextField();
                     tempOnDeckTextFields[i].setFont(windowFont);
                     tempOnDeckTextFields[i].setPreferredSize(tempDim);
                     tempOnDeckTextFields[i].setHorizontalAlignment(JTextField.CENTER);
                     tempOnDeckTextFields[i].setEditable(false);
                     tempOnDeckTextFields[i].setForeground(getForeground());
                     tempOnDeckTextFields[i].setBackground(getBackground());
                }
                onDeckPanel.add(tempOnDeckTextFields[i]);
            }
            onDeckTextFields = tempOnDeckTextFields;

            // redisplay
            c.gridx = 0;
            c.gridy = 0;
            c.anchor = GridBagConstraints.LINE_START;
            c.weightx = 0;
            c.weighty = 0;

            JP.add(headerPanel,c);

            for (int i=0; i<numberOfLanes; i++)  {
                c.gridy = i+1;
                JP.add(lanePanels[i],c);
            }
            c.gridy++;
            JP.add(buttonPanel,c);
            
            c.weighty = 1;
            c.gridy++;
            c.anchor = GridBagConstraints.FIRST_LINE_END;
            JP.add(onDeckPanel, c);

            validate();
        }
    }

    public void setColors(Color frColor, Color bkColor)  {
        setBackground(bkColor);
        setForeground(frColor);
    }
    public void setLaneColors(int lane, Color frColor, Color bkColor)  {
        if((lane > 0) && (lane <= numberOfLanes))  {
            lanePanels[lane-1].setForeground(frColor);
            lanePanels[lane-1].setBackground(bkColor);
            placeLabels[lane-1].setForeground(frColor);
        }
        else if (lane ==0) setColors(frColor, bkColor);
    }
    public void setLaneItemSpacing(int[] sizes)  {
        for (int i=0; i< sizes.length && i<laneItemSpacing.length; i++)
            laneItemSpacing[i] = sizes[i];
        laneHeight = laneItemSpacing[5];
    }
    public void setUseCarNumColor(boolean useColor)  {
        useCarNumColors = useColor;
    }

    public void setLaneTextFieldsOpaque(boolean textFieldsOpaque)  {
        useLaneColorsForTextFields = !textFieldsOpaque;
    }
    public void setLaneBorderSize(int bSize)  {
         headerPanel.setBorder(BorderFactory.
                 createEmptyBorder(bSize,bSize,0,bSize));
         for (int i=0; i<numberOfLanes; i++)
             lanePanels[i].setBorder(BorderFactory.
                     createEmptyBorder(bSize,bSize,bSize,bSize));
    }
    public void setShowTimedField(boolean showTime)  {
        showTimeTextField = showTime;
    }
    public void setShowPlaceCheckBoxes(boolean showPlaceBoxes)  {
        showPlaceCheckBoxes = showPlaceBoxes;
    }
    
    public void setNextHeatButton(JButton nextH)  {
        nextEventButton = nextH;
    }
    public void setNameDisplay(int nd)  {
        nameDisplay = nd;
    }
    public int getCurrentRaceEvent()  {
        return raceEvent;
    }
    public void startRace(int lineUp[][], int event)  {
        raceLineUp = lineUp;
        raceEvent = event;
        
        nextHeatButton.setEnabled(true);
        heatTypeAndNumTextField.setText("Click '>' to Begin");
        PollEntry tempEntry;
        
        // enable checkboxes
        if(showPlaceCheckBoxes)
            for (int i=0; i<numberOfLanes; i++)  {
                for (int j=0; j<numberOfLanes; j++)  {
                    placeCheckBoxes[i][j].setEnabled(true);
                }
            }
        for (int i=0; i<numberOfLanes; i++)  {
            // enable time field
            if (showTimeTextField)
                timeTextFields[i].setEditable(true);

            // show cars on deck
            tempEntry = pollArray.getEntryCopy(raceLineUp[0][i]);
            onDeckTextFields[i].setText("" + tempEntry.carNumber);
            if(useCarNumColors)  {
                onDeckTextFields[i].setBackground(tempEntry.backgroundColor);
                onDeckTextFields[i].setForeground(tempEntry.foregroundColor);
            }

        }
    }
    public void clearRace() {
        currentRaceHeat = -1;
        raceLineUp = null;
        raceEvent = 0;
        nextHeatButton.setEnabled(false);
        previousHeatButton.setEnabled(false);
        heatTypeAndNumTextField.setText("N/A");
        for(int lane = 0; lane < numberOfLanes; lane++)  {
            carNumberTextFields[lane].setText("");
            driverTextFields[lane].setText("");
            timeTextFields[lane].setText("");
            for (int place = 0; place < numberOfLanes; place++)  {
                placeCheckBoxes[lane][place].setOpaque(true);
                placeCheckBoxes[lane][place].setSelected(false);
                placeCheckBoxes[lane][place].setEnabled(false);
            }
            onDeckTextFields[lane].setText("");
        }
        regenerate();
    }

    public String[]  saveRace()  {
        String [] saveData = null;
        String tempString;
        if (raceLineUp !=null)  {

            saveData = new String[raceLineUp.length + 6];
            // save race conditions
            saveData[0] = "Lineup";
            tempString = "RaceEvent, ";
            if (raceEvent== 0)  {
                tempString = tempString + "Preliminary";
            }
            else if (raceEvent == 1)  {
                tempString = tempString + "Finals";
            }
            else {
                tempString = tempString + "None";
            }
            saveData[1] = tempString;

            saveData[2] = "CurrentHeat, " + (currentRaceHeat +1);
//            saveData[2] = "Lanes, " + raceLineUp[0].length;
            saveData[3] = "Heats, " + raceLineUp.length;
            saveData[4] = "";

            // saving lineup
            for (int heat = 0; heat < raceLineUp.length; heat++)  {
                tempString = ""+ raceLineUp[heat][0];
                for (int lane = 1; lane < raceLineUp[0].length; lane++)  {
                    tempString = tempString +","+ raceLineUp[heat][lane];
                }
                saveData[heat+5] = tempString;
            }
            saveData[saveData.length-1] = "End Lineup";
        }

        return saveData;
    }
    public String[]  saveForm()  {
        String [] saveData = null;
        String tempString;
        if (raceLineUp !=null)  {

            saveData = new String[raceLineUp.length + 5];
            // save race conditions
            saveData[0] = "Lineup";
            tempString = "RaceEvent, ";
            if (raceEvent== 0)  {
                tempString = tempString + "Preliminary";
            }
            else if (raceEvent == 1)  {
                tempString = tempString + "Finals";
            }
            else {
                tempString = tempString + "None";
            }
            saveData[1] = tempString;

            saveData[2] = "Heats, " + raceLineUp.length;
            saveData[3] = "";

            // saving lineup
            for (int heat = 0; heat < raceLineUp.length; heat++)  {
                tempString = ""+ String.format("%3d: [%3d],______",(heat+1), raceLineUp[heat][0]);
                for (int lane = 1; lane < raceLineUp[0].length; lane++)  {
                    tempString = tempString +String.format(", [%3d],______", raceLineUp[heat][lane]);
                }
                saveData[heat+4] = tempString;
            }
            saveData[saveData.length-1] = "End Lineup";
        }

        return saveData;
    }
    
    public void loadRace(String[] lineup)  {
        String[] data;

        clearRace();
        // enable checkboxes
        if(showPlaceCheckBoxes)
            for (int i=0; i<numberOfLanes; i++)  {
                for (int j=0; j<numberOfLanes; j++)  {
                    placeCheckBoxes[i][j].setEnabled(true);
                }
            }
        for (int i=0; i<numberOfLanes; i++)  {
            // enable time field
            if (showTimeTextField)
                timeTextFields[i].setEditable(true);
        }
        
        
        try {
            data = lineup[1].split(",");
            for (int i = 0; i < data.length; i++) {
                data[i] = data[i].trim();
            }
            if (data[1].contentEquals("Preliminary")) {
                raceEvent = 0;
                
            } else if (data[1].contentEquals("Finals")) {
                raceEvent = 1;
                
            } else {
                raceEvent = -1;
                
            }
            data = lineup[2].split(",");
            for (int i = 0; i < data.length; i++) {
                data[i] = data[i].trim();
            }
            currentRaceHeat = Integer.parseInt(data[1])-1;

            data = lineup[3].split(",");
            for (int i = 0; i < data.length; i++) {
                data[i] = data[i].trim();
            }
            int numOfHeats = Integer.parseInt(data[1]);
            data = lineup[5].split(",");
            raceLineUp = new int[numOfHeats][data.length];
            for (int heat = 0; heat < numOfHeats; heat++)  {
                data = lineup[heat+5].split(",");
                for (int i=0; i< data.length; i++)  {
                    data[i] = data[i].trim();
                }

                for (int lane = 0; lane < raceLineUp[0].length; lane++) {
                    raceLineUp[heat][lane] = Integer.parseInt(data[lane]);
                }
            }
            nextHeatButton.setEnabled(true);

            if (currentRaceHeat > 0) {
                previousHeatButton.setEnabled(true);
                previousHeatButton.doClick();
            }
            else {
                previousHeatButton.setEnabled(false);
            }
            heatTypeAndNumTextField.setText("Race Loaded");
            
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Error Parsing Lineup:"
                    + e.getMessage());
        }
    }
    
    // action listeners
    private class NextHeatButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            // records scores and displays next heat.
            PollEntry tempEntry;

            // ********************
            // ** Record Scores
            // ********************
            int results[] = new int [numberOfLanes];
            String driverName;
            double resultsTime[] = new double[numberOfLanes];
            if (currentRaceHeat >= 0 && currentRaceHeat < raceLineUp.length) {
                for (int lane = 0; lane < numberOfLanes; lane++)  {
                    results[lane] = numberOfLanes + 1;
                    resultsTime[lane] = 999;
                    if(showTimeTextField)
                        try  {
                            resultsTime[lane] = Double.parseDouble(timeTextFields[lane].getText());
                        }
                        catch (Exception err)  {
                            JOptionPane.showMessageDialog(null, "Error :" +
                                    "Problem with lane entry" +
                                    err.getMessage());
                        }
                    else
                        for (int place=0; place<numberOfLanes; place++)
                            if(placeCheckBoxes[lane][place].isSelected())
                                results[lane] = place+1;

                }
                pollArray.recordRaceResults(currentRaceHeat, raceLineUp[currentRaceHeat], results, resultsTime);
            }

            // Clear Checkboxes and time field
            for (int lane = 0; lane < numberOfLanes; lane++)  {
                for( int place = 0; place < numberOfLanes; place ++)  {
                    placeCheckBoxes[lane][place].setSelected(false);
                    if(!showTimeTextField)
                        placeCheckBoxes[lane][place].setEnabled(true);
                    placeCheckBoxes[lane][place].setOpaque(true);
                }
                timeTextFields[lane].setText("");
            }

            // ********************
            // ** Increment Heats
            // ********************
            if (currentRaceHeat == 0)
                previousHeatButton.setEnabled(true);
            currentRaceHeat ++;
            int nextHeatNum = currentRaceHeat+1;
            if (nextHeatNum > raceLineUp.length)  {
                nextHeatButton.setEnabled(false);
                previousHeatButton.setEnabled(false);
                heatTypeAndNumTextField.setText("N/A");
                for(int lane = 0; lane < numberOfLanes; lane++)  {
                    carNumberTextFields[lane].setText("");
                    driverTextFields[lane].setText("");
                    timeTextFields[lane].setText("");
                    for (int place = 0; place < numberOfLanes; place++)  {
                        placeCheckBoxes[lane][place].setOpaque(true);
                        placeCheckBoxes[lane][place].setSelected(false);
                        placeCheckBoxes[lane][place].setEnabled(false);
                    }
                    onDeckTextFields[lane].setText("");
                }
                currentRaceHeat = -1;
                nextEventButton.doClick();
            }
            else  {
                switch (raceEvent)  {
                    case 0:
                        heatTypeAndNumTextField.setText("Preliminary Heat: " + nextHeatNum +
                                " of " + raceLineUp.length);
                        break;
                    case 1:
                        heatTypeAndNumTextField.setText("Finals Heat: " + nextHeatNum +
                            " of " + raceLineUp.length);
                }

                for (int lane = 0; lane < numberOfLanes && currentRaceHeat >= 0; lane++)  {
                    tempEntry = pollArray.getEntryCopy(raceLineUp[currentRaceHeat][lane]);
                    carNumberTextFields[lane].setText("" + tempEntry.carNumber);
                    driverName = "";
                    if(tempEntry.leader)
                        driverName = "LEADER: ";
                    switch (nameDisplay)  {
                        case 0:
                            driverName = driverName + tempEntry.firstName + " " + tempEntry.lastName;
                            break;
                        case 1:
                            driverName = driverName + tempEntry.firstName.substring(0,1) +
                                    ". " + tempEntry.lastName;
                            break;
                        case 2:
                            driverName = driverName + tempEntry.firstName + " " +
                                    tempEntry.lastName.substring(0, 1) + ".";
                    }
                    driverTextFields[lane].setText(driverName);

                    if (useCarNumColors){
                        carNumberTextFields[lane].setForeground(tempEntry.foregroundColor);
                        carNumberTextFields[lane].setBackground(tempEntry.backgroundColor);
                    }
                    // Set Place Checkboxes
                    if (tempEntry.getPlace(currentRaceHeat) <= numberOfLanes) {
                        placeCheckBoxes[lane][(tempEntry.getPlace(currentRaceHeat)-1)].doClick();
                    }
                    // set time text field
                    timeTextFields[lane].setText("" + tempEntry.getTime(currentRaceHeat));
                    
                    // on Deck
                    if (nextHeatNum < raceLineUp.length) {
                        tempEntry = pollArray.getEntryCopy(raceLineUp[nextHeatNum][lane]);
                        onDeckTextFields[lane].setText("" + tempEntry.carNumber);
                        if (useCarNumColors)  {
                            onDeckTextFields[lane].setForeground(tempEntry.foregroundColor);
                            onDeckTextFields[lane].setBackground(tempEntry.backgroundColor);
                        }
                    }
                    else {
                        onDeckTextFields[lane].setText("");
                        onDeckTextFields[lane].setForeground(getForeground());
                        onDeckTextFields[lane].setBackground(getBackground());
                    }
                }
            }
            if (showTimeTextField)  {
                timeTextFields[0].requestFocus();
                timeTextFields[0].selectAll();
            }
        }
    }
    
    private class PreviousHeatButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            // records scores and displays next heat.
            // **********************
            // ** Record Score
            // *********************
            int results[] = new int [numberOfLanes];
            String driverName;
            double resultsTime[] = new double [numberOfLanes];
            if (currentRaceHeat >= 0 && currentRaceHeat < raceLineUp.length) {
                for (int lane = 0; lane < numberOfLanes; lane++)  {
                    results[lane] = numberOfLanes + 1;
                    resultsTime[lane] = 999;
                    if(showTimeTextField)
                        try  {
                            resultsTime[lane] = Double.parseDouble(timeTextFields[lane].getText());
                        }
                        catch (Exception err)  {
                            // do nothing
                        }
                    else
                        for (int place=0; place<numberOfLanes; place++)
                            if(placeCheckBoxes[lane][place].isSelected())
                                results[lane] = place+1;

                }
                pollArray.recordRaceResults(currentRaceHeat, raceLineUp[currentRaceHeat], results, resultsTime);
            }
            // Clear Checkboxes and time field
            for (int lane = 0; lane < numberOfLanes; lane++)  {
                for( int place = 0; place < numberOfLanes; place ++)  {
                    placeCheckBoxes[lane][place].setSelected(false);
                    if(!showTimeTextField)
                        placeCheckBoxes[lane][place].setEnabled(true);
                    placeCheckBoxes[lane][place].setOpaque(true);
                }
                timeTextFields[lane].setText("");
            }

            // *******************
            // ** Go Back a Heat
            // *******************

            PollEntry tempEntry;
            currentRaceHeat --;
            if (currentRaceHeat <= 0)
                previousHeatButton.setEnabled(false);
            int nextHeatNum = currentRaceHeat+1;
            if (nextHeatNum == raceLineUp.length-1)
                nextHeatButton.setEnabled(true);
            switch (raceEvent)  {
                case 0:
                    heatTypeAndNumTextField.setText("Preliminary Heat: " + (currentRaceHeat+1) +
                            " of " + raceLineUp.length);
                    break;
                case 1:
                    heatTypeAndNumTextField.setText("Finals Heat: " + (currentRaceHeat+1)+
                            " of " + raceLineUp.length);

            }
            
            for (int lane = 0; lane < numberOfLanes; lane++)  {
                tempEntry = pollArray.getEntryCopy(raceLineUp[currentRaceHeat][lane]);
                carNumberTextFields[lane].setText("" + tempEntry.carNumber);
                    driverName = "";
                    if(tempEntry.leader)
                        driverName = "LEADER: ";
                    switch (nameDisplay)  {
                        case 0:
                            driverName = driverName + tempEntry.firstName + " " + tempEntry.lastName;
                            break;
                        case 1:
                            driverName = driverName + tempEntry.firstName.substring(0,1) +
                                    ". " + tempEntry.lastName;
                            break;
                        case 2:
                            driverName = driverName + tempEntry.firstName + " " +
                                    tempEntry.lastName.substring(0, 1) + ".";
                    }
                    driverTextFields[lane].setText(driverName);

                // Set Place Checkboxes
                if (tempEntry.getPlace(currentRaceHeat) <= numberOfLanes) {
                    placeCheckBoxes[lane][(tempEntry.getPlace(currentRaceHeat)-1)].doClick();
                }
               
                if (useCarNumColors){
                    carNumberTextFields[lane].setForeground(tempEntry.foregroundColor);
                    carNumberTextFields[lane].setBackground(tempEntry.backgroundColor);
                }

                // Set time text field
                timeTextFields[lane].setText("" + tempEntry.getTime(currentRaceHeat));

                // on Deck
                if (nextHeatNum < raceLineUp.length) {
                    tempEntry = pollArray.getEntryCopy(raceLineUp[nextHeatNum][lane]);
                    onDeckTextFields[lane].setText("" + tempEntry.carNumber);
                    if (useCarNumColors)  {
                        onDeckTextFields[lane].setForeground(tempEntry.foregroundColor);
                        onDeckTextFields[lane].setBackground(tempEntry.backgroundColor);
                    }
                }
            }
            if (showTimeTextField)  {
                timeTextFields[0].requestFocus();
            }

        }
    }
    
    private class PlaceCheckBoxListener implements ActionListener {
        public void actionPerformed(ActionEvent e)
        {
            int selLn=0, selPl=0;
            boolean isPlaced = false;
            for (int i=0; i< numberOfLanes;i++)
                for(int j=0; j<numberOfLanes; j++)
                    if(placeCheckBoxes[i][j] == e.getSource())
                    {
                        selLn = i;
                        selPl = j;
                    }
            if(placeCheckBoxes[selLn][selPl].isSelected())
                for(int i=0; i<numberOfLanes; i++)
                {
                    if(!placeCheckBoxes[selLn][i].isSelected())
                    {
                        placeCheckBoxes[selLn][i].setEnabled(false);
                        placeCheckBoxes[selLn][i].setOpaque(false);
                    }
                    if(!placeCheckBoxes[i][selPl].isSelected())
                    {
                        placeCheckBoxes[i][selPl].setEnabled(false);
                        placeCheckBoxes[i][selPl].setOpaque(false);
                    }
                }
            else
            {
                 for(int i=0; i<numberOfLanes; i++)
                 {
                     isPlaced = false;
                     for(int j=0; j<numberOfLanes; j++)
                         if(placeCheckBoxes[j][i].isSelected()) isPlaced = true;
                     if(!isPlaced)  {
                         placeCheckBoxes[selLn][i].setEnabled(true);
                         placeCheckBoxes[selLn][i].setOpaque(true);
                     }

                     isPlaced = false;
                     for(int j=0; j<numberOfLanes; j++)
                         if(placeCheckBoxes[i][j].isSelected()) isPlaced = true;
                     if(!isPlaced)  {
                         placeCheckBoxes[i][selPl].setEnabled(true);
                         placeCheckBoxes[i][selPl].setOpaque(true);
                     }
                 }
            }
        }
    }

    private class TimeTextFieldFocusListener implements FocusListener  {

        public void focusGained(FocusEvent e) {
            for (int i=0; i< numberOfLanes; i++)
                if (e.getSource() ==  timeTextFields[i])  {
                    timeTextFields[i].selectAll();
                }
            
        }

        public void focusLost(FocusEvent e) {
            // Do Nothing
        }

    }
}
